/**************************************************************************
program name: ibddrop

      Program to simulate IBD to estimate probs
      The state (X) consists of segregation indicators--

         X determines the IBD pattern among the specified descendant gametes.
         Up to 10 gametes can be scored for IBD

      Two methods of scoring IBD -- 
          (1) by a specified (group of) pattern(s) over windows, and
          (2) by pattern marginally over loci

      Original program modified from jntauto,         Fall 1999, EAT
      New scoring routines addedi                     Fall,1999
      Combined into single program,                   May 2000, EAT

      Cleaning up and reformatting the code.
      Unused variables deleted and pointers initialized.
      Can now be built with MORGAN V2.3 libraries.    August 2000, N.L.

      Modified for Morgan parameter statements and setup by proc_pars()
      and proc_auto()                                 Mar 2002, MAJ
***************************************************************************/

#define PNAM "ibddrop"

#include "autozyg.h"

int   main (int argc, char **argv)
{
   int   j, k, sx, s, jj, t, c, ng, pt, mxl, mxi;
   int **dodo = NULL;                       /* obsolete variable                  */
   double **quant = NULL;                   /* not alloc'd (no quantitative data) */
   double ss;

   int   ngenes;                            /* no. of genes                       */
   int   nfou;                              /* no. of founders                    */
   int   nsw;                               /* no. of switches                    */
   int  *nns;                               /* [nc+1]                             */

   double ****nscore;                       /* (1) [nc][ns][nw][wpt]  ibd counts:
                                             *       ea component, ea scoreset,
                                             *       ea window, ea pattern
                                             * (2) [nc][ns][nl][pt]  ibd counts:
                                             *       ea component, ea scoreset,
                                             *       ea locus, ea pattern         */

   int   nwpt;                              /* (1) no. of patterns                */
   int   nwndw;                             /* (1) no. of windows over loci       */
   int  *gw;                                /* (1) [w]                            */

   int  *gaga;                              /* (2) [g]                            */
   int  *npt;                               /* (2) [g+1]  no. of patterns 
                                               for gametes                        */
   int **index;                             /* (2) [g+1][cd+1]                    */
   int **label;                             /* (2) [g+1][pt+1]                    */
   int **list;                              /* (2) [g+1][pt+1]                    */
   int **count;                             /* (2) [nc][nl]  no. of steps
                                               on ea locus on ea component
                                               (NIt used instead)                 */
   double *sss;                             /* (2) [nl]                           */

   /* where,                               (1)  variables for window over loci
    *                                           (method 1)
    *                                      (2)  variables for pattern over loci 
    *                                           (method 2)
    *                                      w   - no. of loci in window 
    *                                      g   - no. of gametes in largest set
    *                                      nc  - no. pedigree components
    *                                      ns  - no. of gamete sets
    *                                      pt  - no. of patterns
    *                                      nl  - no. of loci
    *                                            (Nloci, which is NMarks + 1)
    *                                      nw  - no. of windows over loci     */

   FILE *pedF, *oSeedF;

   atexit (print_tail);
   PCleanUp = clean_auto;
   atexit (clean_morg);
   print_head (*argv);

   if (!proc_pars (argc, argv, PNAM))   /* process input parameter statements */
      exit (EXIT_FAILURE);

   if (!proc_auto ())                   /* check statements for completeness  */
      exit (EXIT_FAILURE);              /* and consistency                    *
                                         * and prepare controls for ibddrop   */

   if (!proc_seeds ())                  /* set up seed parameters             */
      exit (EXIT_FAILURE);

   if (!proc_fils ())                   /* set up pedigree file parameters    */
      exit (EXIT_FAILURE);

   setSeeds (SAMPL);                    /* initialize seeds for rans()        */

   if (!chk_pedfil ())                  /* check pedigee file for errors      */
      give_up (ERR_qit);

   if (!mak_opedfil ())                 /* write new pedigree file,           */
      give_up (ERR_qit);                /* if there are changes               */

   if (!(pedF = open_ped ()))           /* reopen pedigree file               */
      give_up (ERR_qit);

   alloc_ped ();
   input_ped (pedF, 0, dodo, quant);    /*input pedigree                      */
   fclose (pedF);

   nfou = count_founders ();
   printf (" Number of founders is %d\n\n", nfou);
   printf (" Number of components:  %d\n\n", n_compnts);

   /* allocate xx0, xx1, gg0, gg1, as [NLoci][pedsize+1]  
    * gg0, gg1 are the labelled founder genes, 
    * xx0, xx1 are the maternal and paternal segregation indicators  */

   alloc_xxgg (NLoci - 1, pedsize);

   /* now set and index switches  */

   MaxNsw = 2 * (pedsize - nfou);
   nns = malloc ((n_compnts + 1) * sizeof (int));
   alloc_chk (WHERE, nns);

   nsw = list_switches (NLoci - 1, nns);
   printf (" Number of meiosis switches to sample is %d\n\n", nsw);
   fflush (stdout);

   /* allocate and set arrays of marker and trait positions
    * and recombination fractions  */

   alloc_chromo (NLoci - 1, 0);

   printf (" Simulation recombinations:\n");

   for (sx = FEM; sx <= MAL; ++sx)
   {
      set_chromosomes (NMarks, Marek[sx], 0, &Loc0, &Rec0[sx], sx);

      if (sx == MAL)
         printf ("    M: ");
      if (sx == FEM)
         printf ("    F: ");

      for (j = 0; j < NMarks; ++j)
      {
         if (!(j % 7) && j)
            printf ("\n       ");
         printf ("%3d%s", chromo[0][j], dub2nlz (xecomo[sx][0][j], 7, 3));
      }
      printf ("%3d\n", chromo[0][NMarks]);
   }
   printf ("\n");
   fflush (stdout);

   free_marek ();

   /* check probands for gene scoring  */

   set_probgam (n_compnts);

   if (LocWndw)
   {
      /* alloc and init locus window arrays  */

      nwndw = NLoci + 1 - LocWndw;
      for (j = 0, nwpt = 1; j < LocWndw; ++j)
         nwpt *= 2;

      alloc_wndw (&gw, &nscore, nwndw, nwpt);

      for (c = 0; c < n_compnts; ++c)
         for (s = 0; s < NScSet[c]; ++s)
            for (j = 0; j < nwndw; ++j)
               for (pt = 0; pt < nwpt; ++pt)
                  nscore[c][s][j][pt] = 0.0;
   }
   else
   {
      /* alloc and init ibd pattern arrays  */

      alloc_ibd (&gaga, &index, &label, &list, &npt, &count, &nscore, &sss);

      for (c = 0; c < n_compnts; ++c)
         for (s = 0; s < NScSet[c]; ++s)
            for (j = 0; j < NLoci; ++j)
               for (pt = 0; pt <= PTgg[NPrGam[c][s]]; ++pt)
                  nscore[c][s][j][pt] = 0.0;
   }

   /* realise the initial switches if not already read above */

   realise (NLoci - 1, xecomo, nsw);

   /* set genes (gg0, gg1) from initial switches */

   for (j = 0; j < NLoci; ++j)
      ngenes = set_genes (j);

   if (LocWndw)
   {
      /* count total number of patterns - scoring over loci - 2**window-size  */

      nwpt = 1;
      for (j = 0; j < LocWndw; ++j)
         nwpt *= 2;
      printf (" %d pattern sets are to be scored for each proband gamete "
              "pair\n\n", nwpt);
      printf (" Autozygosity will be scored over %d windows of %d loci\n",
              nwndw, LocWndw);
   }
   else
   {
      /* count total number of patterns */

      for (j = 1; j <= MxGam; ++j)
         npt[j] = index_ibd (j, 0, 0, index[j], label[j], list[j], NULL);

      mxl = pr_npatsets (npt);
   }

   /* now test switches and do MC sampling */

   for (t = 0; t < NIt; ++t)
   {
      realise (NLoci - 1, xecomo, nsw);

      for (j = 0; j < NLoci; ++j)
      {
         jj = chromo[0][j];
         ngenes = set_genes (jj);
      }
      for (c = 0; c < n_compnts; ++c)
      {
         if (LocWndw)
         {
            /* score ibd pattern over window in compt c */

            for (s = 0; s < NScSet[c]; ++s)
               for (j = 0; j < nwndw; ++j)
               {
                  pt = pattern (j, LocWndw, ProbIx[c][s], ProbGam[c][s]);
                  ++nscore[c][s][j][pt];
               }
         }
         else
         {
            /* score ibd pattern over loci in compt c */

            for (j = 0; j < NLoci; ++j)
            {
               jj = chromo[0][j];
               for (s = 0; s < NScSet[c]; ++s)
                  score_genes (jj, NPrGam[c][s], gaga, index[NPrGam[c][s]],
                               list[NPrGam[c][s]], ProbIx[c][s], ProbGam[c][s],
                               nscore[c][s][j]);
            }
         }
      }
   }
   for (c = 0; c < n_compnts; ++c)
   {
      pr_cmpnthead (c);
      pr_probhead ();

      if (LocWndw)
      {
         /* for ea set, print probabilities for ea pattern for ea window  */

         for (s = 0; s < NScSet[c]; ++s)
         {
            /* format and print headings  */

            pr_pgamset (c, s);

            pr_wndwhead (nwndw);

            /* format and print data  */

            for (pt = 0; pt < nwpt; ++pt)
            {
               ss = 0.;
               for (j = 0; j < nwndw; ++j)
                  if (nscore[c][s][j][pt] > ss)
                     ss = nscore[c][s][j][pt];
               if (ss / NIt > 0.0001)
               {
                  binary (pt, LocWndw, gw);
                  pr_wndwprob (c, s, nwndw, pt, gw, nscore);
               }
            }
            printf ("\n\n");
         }
      }
      else
      {
         /* for ea set, print probabilities for ea pattern for ea window  */

         for (s = 0; s < NScSet[c]; ++s)
         {
            /* format and print headings  */

            pr_pgamset (c, s);

            mxi = pr_lochead (c, s, chromo, mxl);

            /* format and print data  */

            ng = NPrGam[c][s];
            for (k = 0; k < npt[ng]; ++k)
            {
               ss = 0.0;
               pt = list[ng][k];
               for (j = 0; j < NLoci; ++j)
               {
                  sss[j] = nscore[c][s][j][pt] / NIt;
                  if (sss[j] > ss)
                     ss = sss[j];
               }
               if (ss > 0.0001)
               {
                  invert_label (label[ng][k], ng, gaga);
                  pr_locprob (c, s, ng, gaga, sss, label[ng][k], mxl, mxi);
               }
            }
            printf ("\n\n");
         }
      }
   }
   fflush (stdout);

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }
   free_chromo (0);
   free_xxgg ();

   if (nns)
   {
      free (nns);
      nns = NULL;
   }
   if (LocWndw)
      free_wndw (&gw, &nscore, nwndw, nwpt);
   else
      free_ibd (&gaga, &index, &label, &list, &npt, &count, &nscore, &sss);

   exit (EXIT_SUCCESS);
}
