
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       genedrop.h:  header file for genedrop.c                       */
/*                    contains #includes for header files for          */
/*                        libraries used                               */
/*                    and #defines, typedefs, variables and            */
/*                        function prototypes for file genedrop.c      */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef GENEDROP_H
#define GENEDROP_H

#define DEFINE_DATA

#include "parsie.h"
#include "pedchk.h"
#include "rans.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif /* DMALLOC */

#define GM 0                    /* grandmother                          */
#define GF 1                    /* grandfather                          */
#define FOUNDER -1              /* founder                              */

FILE * pedF;
FILE * oPedF;
FILE * oSeedF;

                        /*kDat:  maternal & paternal inheritance        */
struct kDat {           
    int kMat;                   /*maternal grandma's or grandpa's allele*/              
    int kPat;                   /*paternal grandma's or grandpa's allele*/
            };                  /* ('GM' or 'GP')                       */
typedef struct kDat KDAT;
typedef KDAT * KDAT_PTR;
KDAT_PTR pKDat;

                        /*genoDat:  genotype for each locus             */
struct genoDat {
    int genoMa;                         /*allele inherited from mother  */
    int genoPa;                         /*allele inherited from father  */
               };
typedef struct genoDat GENO;
typedef GENO * GENO_PTR;
GENO_PTR pGenoDat;

                        /*phenoDat: phenotype for each trait locus      */
struct phenoDat {
    double addi;                        /*additive component            */
    double resi;                        /*residual component            */
    double quant;                       /*quantitative trait value      */
                };
typedef struct phenoDat PHENO;
typedef PHENO * PHENO_PTR;
PHENO_PTR pPhenoDat;

static struct _cleanup clean_drop[] = {
{"ugbmdpal",    free_fnams},
{"ugbmdpal",    freeChkPed},
{"ugbmdpal",    freeKey},
{"    d   ",    free_drop},
{""        ,    NULL}}; 

void     allocGenes(void),      dropMarkerGenes(void), 
         dropTraitGenes(void),  openSimPed(void),      
         writSimPed(void),      openSeedOut(void),   
         freeStuff(void);
         
void     foundGenes(ALLELE_PTR, KDAT_PTR, GENO_PTR, int),
         inheritGenes(KEYS_PP, RMAP_PTR, KDAT_PTR, GENO_PTR, int),
         foundTrait(PHENO_PTR, TPARS_PTR),
         inheritTrait(KEYS_PP, PHENO_PTR, TPARS_PTR, int),
         makQuant(GENO_PTR, PHENO_PTR, TPARS_PTR);

#endif /*GENEDROP_H*/

