
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       globs.h:  header file defining global variables               */
/*                 -  global in scope                                  */
/*                 -  included by main programs and function libraries */
/*                 -  contains modelling or simulation parameters      */
/*                    derived from the input statements                */
/*                 -  values are stored by Pars/ functions             */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  GLOBS_H
#define  GLOBS_H

#ifdef  DEFINE_DATA
GLOBAL char * ProgNam = PNAM;           /* name of morgan program */
#else
GLOBAL char * ProgNam;
#endif /*DEFINE_DATA*/

/***  scalar variables, etc, derived from the parameter statements  ****/

GLOBAL int    PedSize;
GLOBAL int    NNams,        NInts,        NReals,       NFix,         NNInts;
GLOBAL int    NStdIn,       StdIn[5],     NStdOut,      StdOut[5];
GLOBAL int    NLoci,        NMarks,       NTraits,
              NLnkd,        NUnlnkd,      NVary;

#ifdef  DEFINE_DATA                                  /* default values */
GLOBAL int SexIn  = UNDEF;
GLOBAL int SexOut = UNDEF;
GLOBAL int SexOp  = UNDEF;
GLOBAL int ObsIn  = UNDEF;
GLOBAL int ObsOp  = UNDEF;
GLOBAL int ObsOut = UNDEF;
GLOBAL int DiscardReals = UNDEF;
GLOBAL int MarkSeedSet = N;
GLOBAL int TraitSeedSet = N;
GLOBAL int SamplSeedSet = N;
GLOBAL int MarkSeedsOut = N;
GLOBAL int TraitSeedsOut = N;
GLOBAL int SamplSeedsOut = N;
GLOBAL int ChronOut = N;
GLOBAL int AllVarOut = N;
#else
GLOBAL int SexIn;
GLOBAL int SexOut;
GLOBAL int SexOp;
GLOBAL int ObsIn;
GLOBAL int ObsOp;
GLOBAL int ObsOut;
GLOBAL int DiscardReals;
GLOBAL int MarkSeedSet;
GLOBAL int TraitSeedSet;
GLOBAL int SamplSeedSet;
GLOBAL int MarkSeedsOut;
GLOBAL int TraitSeedsOut;
GLOBAL int SamplSeedsOut;
GLOBAL int ChronOut;
GLOBAL int AllVarOut;
#endif  /*DEFINE_DATA*/

GLOBAL uint32 MarkSeedC,    MarkSeedT,    TraitSeedC,   TraitSeedT;
GLOBAL uint32 SamplSeedC,   SamplSeedT;

       enum sEnums {NAM, PA, MA, SEX, OBS};


/*****     the following variables are used by Nghds/ functions     *****/

GLOBAL int NColData;                    /*(No. of integers) + 1 to      */
                                        /*selectively read from ped file*/

GLOBAL int MaxNCmps;                    /*No. of components allocated   */

GLOBAL int SelTrait[FMAX_REAL];         /*Selection/nonselection of     */
                                        /*reals (traits) from ped file  */

GLOBAL int SelInt[FMAX_INT];            /*Selection/nonselection of     */
                                        /*integers from ped file        */


/**** the following variables and tables are used by Quant functions ****/

GLOBAL int FENo[FIXEFF];                /*For fixed effects selected,   */
                                        /*fixed effect #                */

GLOBAL int TrNo[MAX_TR];                /*For traits selected,          */
                                        /*trait #                       */

GLOBAL int FESex;                       /*Gender a fixed effect?        */

GLOBAL int FfDex[MAX_TR][FIXEFF];       /*Presence/absence of           */
                                        /*fixed effects j in trait i    */

GLOBAL int IFix[FIXEFF];                /*Number of levels of fixed     */
                                        /* effects                      */

GLOBAL int IOCor;                       /*flag for printing correlations*/
                                        /*(vs. covariances)             */

GLOBAL int ChkEigC;                     /*flag for printing progress in */
                                        /*computing eigenvalues         */


/*** the following variables are used by Quant/ and PolyEM functions ****/

GLOBAL double Mufti[MAX_FEP][FIXEFF][LEVELS+1];  /*Fixed effect params; */
                                        /*Up to MAX_FEP combinations    */

GLOBAL double SigAdd, SigErr;           /* variance parameters          */
                                        /* (univariate)                 */

GLOBAL double TauSq[MAX_TR*(MAX_TR+1)/2],           /* covariances      */
              SigSq[MAX_TR*(MAX_TR+1)/2];

GLOBAL int MuftiStart[MAX_FEP][FIXEFF]; /* flags for starting values    */
                                        /* given for Mufti[][]          */

GLOBAL double * peigen;


/*** the following structure, _filtab, is used by Stuff/ functions      */
/*   and by other MORGAN functions                                      */

/*  Limits for each program                                             */

struct _prog  {
  char *pnam;                           /*program name                  */
  char pcod;                            /*one char program code         */
  int  min_trait;                       /*minimum no. of traits         */
  int  max_trait;                       /*maximum no. of traits         */
  int  max_fef;                         /*maximum no. of fixed effects  */
  int  max_chrom;                       /*maximum chromosome no.        */
  int  max_mark;                        /*maximum no. of markers        */
  int  max_allele;                      /*maximum no. of alleles        */
  int  max_ped;                         /*maximum pedigree size         */
  int  max_obs;                         /*maximum no. observed          */
  int  max_cmp;                         /*maximum component size        */
  int  max_n_cmp;                       /*maximum number of components  */
               } *pmx;

#ifdef DEFINE_DATA
struct _prog Prog[] = {
 {"univar"  , 'u',  1,         1,   FIXEFF,        0,         0,          0,
                                 MAX_U_PED, MAX_U_OBS,        0,   MAX_NCMP },
 {"unibig"  , 'g',  1,         1,   FIXEFF,        0,         0,          0,
                                   MAX_PED,   MAX_OBS,MAX_CPSIZ,   MAX_NCMP },
 {"bivar"   , 'b',  2,         2,   FIXEFF,        0,         0,          0,
                                   MAX_PED,   MAX_OBS,MAX_CPSIZ,   MAX_NCMP },
 {"multivar", 'm',  1, MAX_TRAIT,   FIXEFF,        0,         0,          0,
                                   MAX_PED,   MAX_OBS,MAX_CPSIZ,   MAX_NCMP },
 {"genedrop", 'd',  0, MAX_SIM_TR,       0,MAX_CHROM,  MAX_MARK,  MAX_ALLEL,
                                   MAX_PED,        0,         0,          0 },
 {"pedcheck", 'p',  0,         0,        0,        0,         0,          0,
                                   MAX_PED,        0,         0,          0 },
 {"kin"     , 'k',  0,         0,        0,        0,         0,          0,
                                   MAX_PED,  MAX_OBS, MAX_CPSIZ,   MAX_NCMP },
 {"lm_auto",  'a',  1, MAX_SIM_TR,        0,MAX_CHROM,  MAX_LOC,  MAX_ALLEL,
                                   MAX_PED,      NOBS,MAX_CPSIZ, MAX_A_NCMP },
 {"lm_lods",  'l',  1, MAX_SIM_TR,        0,MAX_CHROM,  MAX_LOC,  MAX_ALLEL,
                                   MAX_PED,      NOBS,MAX_CPSIZ, MAX_A_NCMP },
 {"ibddrop",  'i',  1, MAX_SIM_TR,        0,MAX_CHROM,  MAX_LOC,  MAX_ALLEL,
                                   MAX_PED,      NOBS,MAX_CPSIZ, MAX_A_NCMP },
 {"schnell",  's',  1, MAX_SIM_TR,  FIXEFF, MAX_CHROM,  MAX_LOC,  MAX_ALLEL,
                                   MAX_PED,      NOBS,MAX_CPSIZ, MAX_A_NCMP },
 {""        ,'\0',  0,         0,        0,        0,         0,         0,
                                         0,        0,         0,         0  }};
#else
extern struct _prog Prog[];
#endif /*DEFINE_DATA*/

#define PAR      "par"
#define PED      "ped"
#define MDAT     "mark"
#define IGEN     "eig"                          /* (replaces "EIGEN")   */
#define PEEL     "peel"
#define SEED     "seed"
#define OPED     "oped"
#define OSCORE   "oscor"
#define OSEED    "oseed"
#define IRESC    "iresc"
#define ORESC    "oresc"

struct _filtab  {
  char *type;                           /*file type (com. line keyword) */
  int command;                          /*flag for named in command line*/
  int param;                            /*flag for named in par statemnt*/
  int done;                             /*flag for file has been read   */
  int parstat;                          /*flag for contains param stmnts*/
  char *pcods;                          /*progs which use this file type*/
  char *fop;                            /*fopen mode "r", "w" or "a"    */
  char *fname;                          /*file name                     */
  char *fdesc;                          /*description of file           */
                };
typedef struct _filtab * FILTAB_PTR;

#ifdef  DEFINE_DATA
  struct _filtab Filtab[] = {
  { PAR   , N, N, N, Y, "pubgmdalsik", "r", NULL, "parameter file"       },
  { PED   , N, N, N, Y, "pubgmdalsik", "r", NULL, "input pedigree file"  },
  { MDAT  , N, N, N, Y, "      als  ", "r", NULL, "marker data file"     },
  { IGEN  , N, N, N, N, "  bgm      ", "r", NULL, "eigenvalue file"      },
  { PEEL  , N, N, N, N, " u         ", "r", NULL, "peeling file"         },
  { SEED  , N, N, N, Y, "     dalsi ", "r", NULL, "seed file"            },
  { OPED  , N, N, N, N, "pubgmdalsik", "w", NULL, "output pedigree file" },
  { OSCORE, N, N, N, N, "      a    ", "w", NULL, "output score file"    },
  { OSEED , N, N, N, N, "     dalsi ", "a", NULL, "output seed file"     },
  { IRESC , N, N, N, N, "      als  ", "r", NULL, "input rescue file"    },
  { ORESC , N, N, N, N, "      als  ", "w", NULL, "output rescue file"   },
  { ""    , N, N, N, N, ""           , "",  NULL, ""                     }};
#else
  extern struct _filtab Filtab[];
#endif  /*DEFINE_DATA*/


/**** the following variables and arrays are used by PedComp programs ***/

#define KIN   0                         /* compute kinship              */
#define INBR  1                         /* compute inbreeding           */
#define INBR2 2                         /* compute 2-locus inbreeding   */

GLOBAL int NKinCmp;                     /* number of components for     */
                                        /* which kinship computations   */
                                        /* are requested                */
GLOBAL int ** NKinNams;                 /* for ea type of computation,  */
                                        /* for ea component, the no. of */
                                        /* names of probands            */
GLOBAL char **** KinNam;                /* for ea computation, for ea   */
                                        /* component, for ea proband,   */
                                        /* the name                     */
GLOBAL int *** KinIx;                   /* for ea computation, for ea   */
                                        /* component, for ea proband,   */
                                        /* the index                    */

GLOBAL int NInbreedRF;                  /* number of recomb fracts      */

GLOBAL double * InbreedRF;              /* recomb fracts for computing  */
                                        /* 2-locus inbreeding coeffs    */

/**** the following variables and tables are used by PolyEM programs ****/

                                        /*See limits.h for default      */
                                        /*values                        */
#ifdef  DEFINE_DATA
GLOBAL int ItMo = ITMO;                 /*Breeding-value iterations     */
GLOBAL int ItEM = ITEM;                 /*Number of EM iterations       */
GLOBAL int ItWt = ITWT;                 /*How often to write out estmts */
GLOBAL int IPtEM = 0;                   /*iem =1 partitioned EM         */
                                        /*iem =0 full EM                */
GLOBAL int IEn = 0;                     /*ien =1 fit environmental model*/
GLOBAL double Diff = DIFF;              /*Convergence criterion for     */
                                        /*breeding values iteration     */
GLOBAL int ICh = 0;                     /*ich =1 -- call bigcheig       */
                                        /*to compute eigenvalues        */
GLOBAL int ChkEig = N;                  /*flag for printing eigenvals   */
GLOBAL int ChkGnv = N;                  /*flag for printing G-iverse    */
GLOBAL int ChkGmt = N;                  /*flag for printing G-matrix    */
GLOBAL int ChkTrc = N;                  /*flag for printing trace       */

#else
GLOBAL int ItMo;
GLOBAL int ItEM;
GLOBAL int ItWt;
GLOBAL int IPtEM;
GLOBAL int IEn;
GLOBAL double Diff;
GLOBAL int ICh;
GLOBAL int ChkEig;
GLOBAL int ChkGnv;
GLOBAL int ChkGmt;
GLOBAL int ChkTrc;

#endif /*DEFINE_DATA*/


GLOBAL int    TauModel[MAX_TR*(MAX_TR+1)/2],            /* fit/no fit   */
              SigModel[MAX_TR*(MAX_TR+1)/2];            /* covariances  */
                                                        /* (multivar)   */

/******     the following structures are used by genedrop          ******/

                        /* macros for recombMap->recombTyp below        */

#define FM 1                    /* first marker (on chromosome)         */
#define OM 2                    /* other than first marker              */
#define FT 3                    /* trait is first locus on chromosome   */
#define IT 4                    /* intermediate trait - has preceding   */
                                /* marker/trait locus & following marker*/
#define ET 5                    /* trait with no following markers      */
#define UT 6                    /* unlinked trait                       */

                        /*recombMap:  identifies neighbors and          */
                        /*            gender-specific probabilities     */
                        /*            used in recombination algorithms  */
struct recombMap {
    int id;                             /* markers & traits are in order*/
    int recombTyp;                      /* of chrom, locus;  unlinked   */
    int iA;                             /* traits are last              */
    int iB;
    double probA[2];
    double probB[2];
                 };
typedef struct recombMap RMAP;
typedef RMAP * RMAP_PTR;
GLOBAL RMAP_PTR PRecombMap;

                        /*traitPars:  parameters used for computing     */
                        /*            the additive and residual         */
                        /*            of the phenotype                  */
struct traitPars {
    int id;                             /* ids are in recombMap order   */
    double means[3];
    double addi_term_found;
    double addi_term;
    double resi_term;
                 };
typedef struct traitPars TPARS;
typedef TPARS * TPARS_PTR;
GLOBAL TPARS_PTR PTraitPars;

                        /*traitOut:  traits in numerical order          */
struct traitOut {
    int id;
    int wid;
    int dec;
    int ixG;
    int ixPh;
                };
typedef struct traitOut TOUT;
typedef TOUT * TOUT_PTR;
GLOBAL TOUT_PTR PTraitOut;

                        /*alleleFreq:  allele frequencies for each locus*/
struct alleleFreq{
    int nAlleles;                       /*no. of alleles at this locus  */
    double * pCumFreq;                  /*ptr to list of cumulative frqs*/
                 };
typedef struct alleleFreq ALLELE;
typedef ALLELE * ALLELE_PTR;
GLOBAL ALLELE_PTR PAlleleFreq;

GLOBAL int WidAlle;                     /*no. characters for printing   */
                                        /*alleles                       */


/**** the following variables and tables are used by Autozyg programs ***/

#ifdef  DEFINE_DATA
GLOBAL int NIt  = NIT;
GLOBAL int WIt  = WIT;
GLOBAL int FWIt = FWIT;
GLOBAL int ScIt = 1;                    /* score ECLLD and recomb counts*/
                                        /* every SCIT steps or scans    */
GLOBAL int Scan = UNDEF;                /* sample by scan (T) or single */
                                        /* step(F)                      */
GLOBAL int SmplSetup = UNDEF;           /* which sample setup to use    */
GLOBAL int TraitPheno = UNDEF;          /* T for phenotyp, 
                                           F for genotyp data           */
GLOBAL int LocWndw = 0;                 /* window size (no. loci)       */
                                        /* for ibddrop                  */ 
GLOBAL double LSamplProb = 0.;          /* locus sampler probability    */

#else
GLOBAL int NIt;
GLOBAL int WIt;
GLOBAL int FWIt;
GLOBAL int ScIt;
GLOBAL int Scan;
GLOBAL int SmplSetup;
GLOBAL int TraitPheno;
GLOBAL int LocWndw;
GLOBAL double LSamplProb;
#endif /*DEFINE_DATA*/

                              /* no. of positions for trait on chrom    */
GLOBAL int NTPos;             /* for lods scoring                       */

                              /* ADDED BY AWG   */
GLOBAL int NTExtPos;          /* no. of externally specified test pos   */
GLOBAL int NTIntPos;          /* no. of internally specified test pos   */

                              /* no. of alleles at each locus           */
                              /* NAll[0]...NAll[Nloci-1]                */
GLOBAL int * NAll;            /* ordered by name (chromo[][])           */

                              /* allele frequencies at each locus       */
                              /* Qq[0][1]....Qq[0][NAll[0]]             */
GLOBAL double **Qq;           /*          ...Qq[NLoci-1][Nall[NLoci-1]] */

GLOBAL int NObs;              /* no. observed                           */
GLOBAL char ** ObsNam;        /* names of observed individuals          */
GLOBAL int  *  ObsIx;         /* indices of observed individuals        */
GLOBAL int *** ObsMark;       /* for ea observed individual, for ea     */
                              /* marker, a pair of alleles              */

GLOBAL int NScCmp;            /* number of components with proband      */
                              /* gamete sets                            */
GLOBAL int * NScSet;          /* for each component, the number of      */
                              /* of sets of proband gametes             */
GLOBAL int ** NPrGam;         /* for each component for each set,       */
                              /* number of probands                     */
GLOBAL char **** ProbNam;     /* for each component for each set,       */
                              /* the list of names of individuals       */
                              /* corresponding to the proband gametes   */
                              /* to be scored                           */
GLOBAL int *** ProbIx;        /* the list of indices to the individuals */
GLOBAL int *** ProbGam;       /* The list of gametes                    */
GLOBAL int MxGam;             /* The largest number of gametes in a set */

GLOBAL double ** Marek;       /* marker recombination fractions         */
                              /* 1st subscript:  0 -- female; 1 -- male */
                              /* 2nd subscript:                         */
                              /*   Marek[][0] -- .5 (not used)          */
                              /*   Marek[][1] -- recomb between marker  */
                              /*                 "1" and marker "2"     */
                              /*   Marek[][n] -- recomb between marker  */
                              /*                 "n" and marker "n+1"   */
                              /*   Marek[][NMarks] -- .5                */

GLOBAL int Loc0;              /* position for linked trait              */
GLOBAL double Rec0[2];        /* and gender-specific recomb fractions   */

GLOBAL int * IntVal;          /* [NTPos+1] positions for trait          */
GLOBAL double ** RecVal;      /* [2][NTPos+1] sex-specifc recomb fracts */

#endif  /*GLOBS_H*/
