/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*
 *       msample.h:  header file for the Msample/ library                     * 
 *                -  included by Msample/ (meiosis sampling) functions        *
 *                   and by main programs calling Msample/ functions          * 
 *::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef MSAMPLE_H
#define MSAMPLE_H

#include "incs.h"
#include "nghds.h"
#include "sample.h"
#include "rans.h"

typedef struct _Switch
{
   Ind   ind;
   int   sex;
}
Switch;

typedef struct _Switchlist
{
   struct _Switch *current;
   struct _Switchlist *next;
}
Switchlist;

typedef struct _Switchcmp
{
   int   index;
   struct _Switchlist *meioses;
}
Switchcmp;

struct gp_rfunc {
        struct gp_rfunc *next;
        int *inv;
        double *p;
        int n_inv,nc,flag;
};

struct gp_rfunc_ptr {
        struct gp_rfunc_ptr *next;
        struct gp_rfunc *rf;
};

struct gp_rfnode {
        struct gp_rfnode *next;
        struct gp_rfunc *rf;
        int y;
};

struct deg_list {
        struct deg_list *next,*prev,*abs_list;
        int deg,gene;
};


/* Maximum number of genes in an RFunction for gen_pen()
 * Note that memory and patience will run out long before this does...       */

#define MAX_GP_RF_GENES 64

GLOBAL int MaxNsw;            /* maximum no. of switches = 
                               *                  2 * pedsize - nfounders    */
GLOBAL Switch *meiosis;       /* vector of the meioses of the pedigree       */
GLOBAL Switchcmp *swchlist;   /* linked lists of the meioses, by component   */

GLOBAL int ***lpflag;         /* [2][nh+1][NMarks]  flags set by set_lpflag
                               * to indicate when log-likelihoods may differ
                               * and must be included in probability ratio
                               * (note: NLoci is NMarks + NTraits)           */

int   read_gen_dat (FILE *, int **, int **, int *);
/*void  allocate (int, int);*/
void  set_switches (int, int, int *, int *);
void  read_segs (FILE *, int, int, int);
int   list_switches (int, int *);
void  free_switches (void);
int   only_kid (Ind *, Ind *);
void  set_chromosomes (int, double *, int, int *, double *, int);
void  set_lpflag (int, int, int);
void  invert_label (int, int, int *);
int   index_ibd (int, int, int, int *, int *, int *, FILE *);
void  get_genes (int, int, int *, int *, int *);
int   score_ibd (int, int *);
void  gene_update (int, int, int);
void  fwd_compute (int, double, double *, double *, double *);
int   back_sample (int, int, int, double, double);
void  realise (int, double ***, int);
void  trait_segs (int);
void  next_marker_segs (int, int, int, int);
void  set_genotypes (int, int, double *, int *, int *);
void  print_segs (int);
void  print_set_up (int);
void  print_obs_genes (int, int ***);
double new_pen (int, int *[2], int ***, double *, int, int, int *);
double prob_data (int, int, int, int);
int   score_either (int, int, int);
int   score_whichever (int, int, int, int);
void  free_allocate_loci (void);
/*void  free_allocate (void);*/
void  free_allefrq (void);
void  print_chromosome (int, int, int);
void  do_msmpl (int, int, int, int, double **, int ***, int);
void  full_seg_prob (int, int, int ***, double **);
double diff_seg_prob (int, int, int, int *, double **);
double lik_rat (int, int, int, int);
double full_pen_seg_lod (int, int, int, int, int ***);
void  score_genes (int, int, int *, int *, int *, int *, int *, double *);
int   pattern (int, int, int *, int *);
void  binary (int, int, int *);

/* ** new from Simon  06/20/01 ** */
double gen_pen(int,int *[2],int ***,double *,int,int,int *);
void alloc_gen_pen(int,int);
void free_gen_pen(void);
void alloc_seg_pen(int,int,int ***, double ***);
void free_seg_pen(int, double ***);
double seg_pen(int,int *[2],int ***,double *,int,int,int *);
double seg_pen2(int,int ***,double *,int,int,int *);
/* ** new MAJ  08/01 ** */
void  alloc_chromo (int, int),        free_chromo (int);
void  alloc_lpflag (int, int),        free_lpflag (int);
void  alloc_logprob (void),           free_logprob (void);
void  alloc_msmpl (void),             free_msmpl (void);
void  ali_nc      (int **),           ali_nl      (int **),
      ali_n_nl    (int ***, int),     ali_nc_n    (int ***, int),
      ali_nl_n    (int ***, int),     ali_nc_nl   (int ***),
      ali_nc_nl_n (int ****, int);    
void  ald_nl      (double **),        ald_nc_n    (double ***, int),
      ald_nl_n    (double ***, int),  ald_nc_nl   (double ***),
      ald_nc_nl_ns(double ****);
void  fre_n       (void **),          fre_n_n     (void ***, int),
      fre_nc_n    (void ***),         fre_nl_n    (void ***),
      fre_nc_nl_n (void ****);
/* following moved from Autozyg (mostly)  EAT  feb 2002 */
void  alloc_ibd   (int **, int ***, int ***, int ***, int **, 
                   int ***, double *****, double **);
void  alloc_wndw  (int **, double *****, int, int);
void  alloc_loglik(int ****, double ***, double ***, double ***);
void  alloc_lod   (int **, int ***, double ***,
                   double ***, double ***, double ***, double ***);
void  free_ibd    (int **, int ***, int ***, int ***, int **, 
                   int ***, double *****, double **);
void  free_cst    (void);
void  free_wndw   (int **, double *****, int, int);
void  free_loglik (int ****, double ***, double ***, double ***);
void  free_lod    (int **, int ***, double ***, double ***, double ***,
                   double ***, double ***);
void  set_traitdat(int ***, int *, int *, int, int);
void  set_probgam (int);
/* print functions for lm_auto and ibddrop */
int   pr_npatsets (int *);
void  pr_pgamset  (int, int);
void  pr_cmpnthead(int);
void  pr_probhead (void);
int   pr_lochead  (int, int, int **, int);
void  pr_locprob  (int, int, int, int *, double *, int, int, int);
void  pr_wndwhead (int);
void  pr_wndwprob (int, int, int, int, int *, double ****);

void    alloc_xxgg  (int, int);
void    free_xxgg   (void);
int *** alloc_y     (int, int);
void    free_y      (int ****);
int  ** alloc_genes (int, int);
void    free_genes  (int ***);
 
/* no longer current *    04/02           */
/*void  allocz_y    (int ****, int, int); */
/*void  free_y      (int ****, int);      */

#endif
