
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       parsedefs.h:  header file for the Pars/ library               */
/*                     -  of file scope                                */
/*                     -  included by Pars/ functions                  */
/*                     -  contains definitions of parameter statements */
/*                        and command line paramters for main programs */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  PARSEDEFS_H
#define  PARSEDEFS_H

#define KEYWD 0                         /*Token is a keyword            */
#define PDAT 1                          /*Token is data                 */
#define MAXNKEY  8                      /*The max # of keywords in a    */
                                        /*statement                     */
#define MAXNTOKEN 200                   /*The max # of tokens in a      */
                                        /*statement (init. # only)      */
#define NOMTOKLEN  8                    /*Nominal # of characters per   */
                                        /*token in an input statement   */

#define COM_LIN_FORM  "<prognam> <parfile>"

/*  line_in[]   - the next line of input data                           */
/*  token[][]   - the tokens found in the input statemnt                */
/*  token[]     - a list of pointers to the start of each token         */
/*  tok_typ[]   - corresponding token type                              */
/*                (alphabetic, numeric or string)                       */
/*  aggre_key[] - the aggregate key                                     */
/*                (concatenated first 4 characters of each keyword)     */
/*  key_ix      - the statement type, as determined by the aggregate key*/

static char     line_in[MAXLINLEN+2];
static char  ** token;
static int    * tok_typ;
static char     aggre_key[MAXNKEY*4+1];
static int      key_ix;

/*  function prototypes for functions which correspond to the various   */
/*  input statement types                                               */

static int
    not_impl       (void),
    allow_cmpsiz   (void),
    allow_obsiz    (void),
    allow_pedsiz   (void),
    assign_gend    (void),
    assum_all_obs  (void),
    check_eigen    (void),
    check_eig_cmp  (void),
    check_ginv     (void),
    check_gmat     (void),
    check_mc_iter  (void),
    check_trace    (void),
    cmpu_cinbreed  (void),
    cmpu_ckinshp   (void),
    cmpu_c2_loc    (void),
    comp_eigen     (void),
    cmpu_inbreed   (void),
    cmpu_kinshp    (void),
    cmpu_2_loc     (void),
    fit_addi       (void),
    fit_no_gen     (void),
    fit_resi       (void),
    ignor_gend     (void),
    in_eig_fil     (void),
    in_mrkdat_fil  (void),
    in_gend_abs    (void),
    in_gend_pres   (void),
    in_ord_fath    (void),
    in_ord_moth    (void),
    in_obs_abs     (void),
    in_obs_pres    (void),
    in_ped_fil     (void),
    in_ped_int     (void),
    in_ped_int_rl  (void),
    in_ped_n       (void),
    in_ped_n_i     (void),
    in_ped_n_i_r   (void),
    in_ped_n_r     (void),
    in_pedsiz      (void),
    in_peel_fil    (void),
    in_resc_fil    (void),
    in_seed_fil    (void),
    label_mark     (void),
    label_trait    (void),
    lim_breed      (void),
    lim_em_iter    (void),
    map_cmarkd     (void),
    map_cmarkd_f   (void),
    map_cmarkd_m   (void),
    map_cmarkp     (void),
    map_cmarkp_f   (void),
    map_cmarkp_m   (void),
    map_cmarkr     (void),
    map_cmarkr_f   (void),
    map_cmarkr_m   (void),
    map_ctraitd    (void),
    map_ctraitd_f  (void),
    map_ctraitd_m  (void),
    map_ctraitr    (void),
    map_ctraitr_f  (void),
    map_ctraitr_m  (void),
    map_ctr_extd_f (void),
    map_ctr_extd_m (void),
    map_ctr_extd   (void),
    map_ctr_extr_f (void),
    map_ctr_extr_m (void),
    map_ctr_extr   (void),
    map_ctr_intv_f (void),
    map_ctr_intv_m (void),
    map_ctr_intv   (void),
    map_markd      (void),
    map_markd_f    (void),
    map_markd_m    (void),
    map_markp      (void),
    map_markp_f    (void),
    map_markp_m    (void),
    map_markr      (void),
    map_markr_f    (void),
    map_markr_m    (void),
    map_traitd     (void),
    map_traitd_f   (void),
    map_traitd_m   (void),
    map_traitr     (void),
    map_traitr_f   (void),
    map_traitr_m   (void),
    map_tr_extd_f  (void),
    map_tr_extd_m  (void),
    map_tr_extd    (void),
    map_tr_extr_f  (void),
    map_tr_extr_m  (void),
    map_tr_extr    (void),
    map_tr_intv_f  (void),
    map_tr_intv_m  (void),
    map_tr_intv    (void),
    out_all        (void),
    out_chron      (void),
    out_corr       (void),
    out_covar      (void),
    out_mark_seeds (void),
    out_ped_fil    (void),
    out_ord_fath   (void),
    out_ord_moth   (void),
    out_resc_fil   (void),
    out_scor_fil   (void),
    out_scor_iter  (void),
    out_seed_fil   (void),
    out_space      (void),
    out_trait_seeds(void),
    sampl_by_scan  (void),
    sampl_by_step  (void),
    sel_trait      (void),
    sel_tr_eff     (void),
    set_addi_var   (void),
    set_br_conv    (void),
    set_cloc_wind  (void),
    set_cmark_dat  (void),
    set_cmark_frq  (void),
    set_cmp_prbgam (void),
    set_eigen      (void),
    set_geno_mean  (void),
    set_lsmpl_prob (void),
    set_loc_wind   (void),
    set_mark_dat   (void),
    set_mark_frq   (void),
    set_mark_seeds (void),
    set_mc_iter    (void),
    set_prbgam     (void),
    set_reco_frq   (void),
    set_resi_var   (void),
    set_sampl_seeds(void),
    set_scor_iter  (void),
    set_trait_frq  (void),
    set_trait_seeds(void),
    set_trait_geno (void),
    set_trait_pheno(void),
    sim_cmark      (void),
    sim_cmark_trait(void),
    sim_mark       (void),
    sim_mark_trait (void),
    sim_unlinked   (void),
    start_add_cor  (void),
    start_add_cov  (void),
    start_add_var  (void),
    start_effect   (void),
    start_mean     (void),
    start_res_cov  (void),
    start_res_cor  (void),
    start_res_var  (void),
    titl_top       (void),
    use_lsmpl_setup(void),
    use_sqimp_setup(void),
    use_full_em    (void),
    use_part_em    (void);

/*  prototype for input statement                                             */
/*  proto.key[]:  string containing the first four letters of each            */
/*                keyword in the statement.                                   */
/*  proto.par:    a code for each data value or list of values in the         */
/*                statement.                                                  */
/*                "i" - an integer value             example:   3000          */
/*                "k" - a list of integer values                2 4 5         */
/*                "h" - a hexadecimal value                     0x73331ad7    */
/*                "r" - a real value                            22.33         */
/*                "x" - a list of real values                   .3 .2 .9      */
/*                "s" - a string                                "pedFil3"     */
/*                "n" - a list of names                         A313 8333     */
/*                "g" - alternating names & integers                          */
/*                      (e.g. proband gametes)                  C16 0 D26 1   */
/*                "a" - sets of name plus list of integers,                   */
/*                      with the no. of sets specified by "i"                 */
/*                      (e.g. marker data)                      A1 3 5 3 3    */
/*                                                              B3 3 2 3 3    */

/*  proto.func:   the name of the function which acts upon the data           */
/*                from the statement                                          */
/*  proto.count:  the number of times this statement type occurs in the       */
/*                input file                                                  */

static char *nam_cod = "nga";   /* codes for data lists containing names      */

static struct _proto {
       char * key;
       char * par;
       char * pcods;
       int (* func) (void);
       int    count;
               } *ppp;

static struct _proto  proto[] = {
{"allo""comp""size"                              ,"i"    ,"ugbm       ",allow_cmpsiz   ,0},
{"allo""obse""indi"                              ,"i"    ,"ugbm       ",allow_obsiz    ,0},
{"allo""pedi""size"                              ,"i"    ,"ugbmdpalsik",allow_pedsiz   ,0},
{"assi""gend"                                    ,""     ,"ugbmdpalsik",assign_gend    ,0},
{"assu""all" "obse"                              ,""     ,"    dp     ",assum_all_obs  ,0},
{"chec""eige"                                    ,""     ,"ugbm       ",check_eigen    ,0},
{"chec""eige""comp"                              ,""     ,"ugbm       ",check_eig_cmp  ,0},
{"chec""ginv"                                    ,""     ," gbm       ",check_ginv     ,0},
{"chec""gmat"                                    ,""     ,"u          ",check_gmat     ,0},
{"chec""prog""mc"  "iter"                        ,"i"    ,"      als  ",check_mc_iter  ,0},
{"chec""trac"                                    ,""     ,"ugbm       ",check_trace    ,0},
{"comp""comp""inbr""coef"                        ,"in"   ,"          k",cmpu_cinbreed  ,0},
{"comp""comp""kins""coef"                        ,"in"   ,"          k",cmpu_ckinshp   ,0},
{"comp""comp""two-""inbr""coef"                  ,"in"   ,"          k",cmpu_c2_loc    ,0},
{"comp""eige"                                    ,""     ," bgm       ",comp_eigen     ,0},
{"comp""inbr""coef"                              ,"n"    ,"          k",cmpu_inbreed   ,0},
{"comp""kins""coef"                              ,"n"    ,"          k",cmpu_kinshp    ,0},
{"comp""two-""inbr""coef"                        ,"n"    ,"          k",cmpu_2_loc     ,0},
{"fit" "addi""corr"                              ,"k"    ,"   m       ",fit_addi       ,0},
{"fit" "addi""cova"                              ,"k"    ,"   m       ",fit_addi       ,0},
{"fit" "envi""mode"                              ,""     ,"   m       ",fit_no_gen     ,0},
{"fit" "resi""corr"                              ,"k"    ,"   m       ",fit_resi       ,0},
{"fit" "resi""cova"                              ,"k"    ,"   m       ",fit_resi       ,0},
{"igno""gend"                                    ,""     ,"ugbmdpalsik",ignor_gend     ,0},
{"inpu""eige""file"                              ,"s"    ," gbm       ",in_eig_fil     ,0},
{"inpu""mark""data""file"                        ,"s"    ,"      als  ",in_mrkdat_fil  ,0},
{"inpu""pedi""file"                              ,"s"    ,"ugbmdpalsik",in_ped_fil     ,0},
{"inpu""pedi""reco""fath""moth"                  ,""     ,"ugbmdpalsik",in_ord_fath    ,0},
{"inpu""pedi""reco""gend""abse"                  ,""     ,"ugbmdpalsik",in_gend_abs    ,0},
{"inpu""pedi""reco""gend""pres"                  ,""     ,"ugbmdpalsik",in_gend_pres   ,0},
{"inpu""pedi""reco""inte"                        ,"i"    ,"    dpalsik",in_ped_int     ,0},
{"inpu""pedi""reco""inte""real"                  ,"ii"   ,"ugbmdpalsik",in_ped_int_rl  ,0},
{"inpu""pedi""reco""name"                        ,"i"    ,"    dpalsik",in_ped_n       ,0},
{"inpu""pedi""reco""name""inte"                  ,"ii"   ,"    dpalsik",in_ped_n_i     ,0},
{"inpu""pedi""reco""name""inte""real"            ,"iii"  ,"ugbmdpalsik",in_ped_n_i_r   ,0},
{"inpu""pedi""reco""name""real"                  ,"ii"   ,"ugbmdpalsik",in_ped_n_r     ,0},
{"inpu""pedi""reco""moth""fath"                  ,""     ,"ugbmdpalsik",in_ord_moth    ,0},
{"inpu""pedi""reco""obse""abse"                  ,""     ,"    dp     ",in_obs_abs     ,0},
{"inpu""pedi""reco""obse""pres"                  ,""     ,"    dp     ",in_obs_pres    ,0},
{"inpu""pedi""size"                              ,"i"    ,"ugbmdpalsik",in_pedsiz      ,0},
{"inpu""peel""sequ""file"                        ,"s"    ,"u          ",in_peel_fil    ,0},
{"inpu""resc""file"                              ,"s"    ,"      al   ",in_resc_fil    ,0},
{"inpu""seed""file"                              ,"s"    ,"    d alsi ",in_seed_fil    ,0},
{"labe""chro""mark"                              ,"iis"  ,"           ",not_impl       ,0},
{"labe""mark"                                    ,"is"   ,"           ",not_impl       ,0},
{"labe""trai"                                    ,"is"   ,"           ",not_impl       ,0},
{"limi""bree""iter"                              ,"i"    ,"ugbm       ",lim_breed      ,0},
{"limi""em"  "iter"                              ,"i"    ,"ugbm       ",lim_em_iter    ,0},
{"map" "chro""gend""f"   "mark""dist"            ,"ix"   ,"    d alsi ",map_cmarkd_f   ,0},
{"map" "chro""gend""f"   "mark""posi"            ,"ix"   ,"    d alsi ",map_cmarkp_f   ,0},
{"map" "chro""gend""f"   "mark""reco""frac"      ,"ix"   ,"    d alsi ",map_cmarkr_f   ,0},
{"map" "chro""gend""f"   "trai""exte""dist"      ,"iix"  ,"       ls  ",map_ctr_extd_f ,0},
{"map" "chro""gend""f"   "trai""exte""reco""frac","iix"  ,"       ls  ",map_ctr_extr_f ,0},
{"map" "chro""gend""f"   "trai""inte""prop"      ,"iix"  ,"       ls  ",map_ctr_intv_f ,0},
{"map" "chro""gend""f"   "trai""mark""dist"      ,"ikkx" ,"    d a  i ",map_ctraitd_f  ,0},
{"map" "chro""gend""f"   "trai""mark""reco""frac","ikkx" ,"    d a  i ",map_ctraitr_f  ,0},
{"map" "chro""gend""m"   "mark""dist"            ,"ix"   ,"    d alsi ",map_cmarkd_m   ,0},
{"map" "chro""gend""m"   "mark""posi"            ,"ix"   ,"    d alsi ",map_cmarkp_m   ,0},
{"map" "chro""gend""m"   "mark""reco""frac"      ,"ix"   ,"    d alsi ",map_cmarkr_m   ,0},
{"map" "chro""gend""m"   "trai""exte""dist"      ,"iix"  ,"       ls  ",map_ctr_extd_m ,0},
{"map" "chro""gend""m"   "trai""exte""reco""frac","iix"  ,"       ls  ",map_ctr_extr_m ,0},
{"map" "chro""gend""m"   "trai""inte""prop"      ,"iix"  ,"       ls  ",map_ctr_intv_m ,0},
{"map" "chro""gend""m"   "trai""mark""dist"      ,"ikkx" ,"    d a  i ",map_ctraitd_m  ,0},
{"map" "chro""gend""m"   "trai""mark""reco""frac","ikkx" ,"    d a  i ",map_ctraitr_m  ,0},
{"map" "chro""mark""dist"                        ,"ix"   ,"    d alsi ",map_cmarkd     ,0},
{"map" "chro""mark""posi"                        ,"ix"   ,"    d alsi ",map_cmarkp     ,0},
{"map" "chro""mark""reco""frac"                  ,"ix"   ,"    d alsi ",map_cmarkr     ,0},
{"map" "chro""trai""exte""dist"                  ,"iix"  ,"       ls  ",map_ctr_extd   ,0},
{"map" "chro""trai""exte""reco""frac"            ,"iix"  ,"       ls  ",map_ctr_extr   ,0},
{"map" "chro""trai""inte""prop"                  ,"iix"  ,"       ls  ",map_ctr_intv   ,0},
{"map" "chro""trai""mark""dist"                  ,"ikkx" ,"    d a  i ",map_ctraitd    ,0},
{"map" "chro""trai""mark""reco""frac"            ,"ikkx" ,"    d a  i ",map_ctraitr    ,0},
{"map" "gend""f"   "mark""dist"                  ,"x"    ,"    d alsi ",map_markd_f    ,0},
{"map" "gend""f"   "mark""posi"                  ,"x"    ,"    d alsi ",map_markp_f    ,0},
{"map" "gend""f"   "mark""reco""frac"            ,"x"    ,"    d alsi ",map_markr_f    ,0},
{"map" "gend""f"   "trai""exte""dist"            ,"ix"   ,"       ls  ",map_tr_extd_f  ,0},
{"map" "gend""f"   "trai""exte""reco""frac"      ,"ix"   ,"       ls  ",map_tr_extr_f  ,0},
{"map" "gend""f"   "trai""inte""prop"            ,"ix"   ,"       ls  ",map_tr_intv_f  ,0},
{"map" "gend""f"   "trai""mark""dist"            ,"kkx"  ,"    d a  i ",map_traitd_f   ,0},
{"map" "gend""f"   "trai""mark""reco""frac"      ,"kkx"  ,"    d a  i ",map_traitr_f   ,0},
{"map" "gend""m"   "mark""dist"                  ,"x"    ,"    d alsi ",map_markd_m    ,0},
{"map" "gend""m"   "mark""posi"                  ,"x"    ,"    d alsi ",map_markp_m    ,0},
{"map" "gend""m"   "mark""reco""frac"            ,"x"    ,"    d alsi ",map_markr_m    ,0},
{"map" "gend""m"   "trai""exte""dist"            ,"ix"   ,"       ls  ",map_tr_extd_m  ,0},
{"map" "gend""m"   "trai""exte""reco""frac"      ,"ix"   ,"       ls  ",map_tr_extr_m  ,0},
{"map" "gend""m"   "trai""inte""prop"            ,"ix"   ,"       ls  ",map_tr_intv_m  ,0},
{"map" "gend""m"   "trai""mark""dist"            ,"kkx"  ,"    d a  i ",map_traitd_m   ,0},
{"map" "gend""m"   "trai""mark""reco""frac"      ,"kkx"  ,"    d a  i ",map_traitr_m   ,0},
{"map" "mark""dist"                              ,"x"    ,"    d alsi ",map_markd      ,0},
{"map" "mark""posi"                              ,"x"    ,"    d alsi ",map_markp      ,0},
{"map" "mark""reco""frac"                        ,"x"    ,"    d alsi ",map_markr      ,0},
{"map" "trai""exte""dist"                        ,"ix"   ,"       ls  ",map_tr_extd    ,0},
{"map" "trai""exte""reco""frac"                  ,"ix"   ,"       ls  ",map_tr_extr    ,0},
{"map" "trai""inte""prop"                        ,"ix"   ,"       ls  ",map_tr_intv    ,0},
{"map" "trai""mark""dist"                        ,"kkx"  ,"    d a  i ",map_traitd     ,0},
{"map" "trai""mark""reco""frac"                  ,"kkx"  ,"    d a  i ",map_traitr     ,0},
{"outp""mark""seed""only"                        ,""     ,"    d      ",out_mark_seeds ,0},
{"outp""pedi""chro"                              ,""     ,"    dp     ",out_chron      ,0},
{"outp""pedi""file"                              ,"s"    ,"ugbmdpalsik",out_ped_fil    ,0},
{"outp""pedi""reco""fath""moth"                  ,""     ,"    dp     ",out_ord_fath   ,0},
{"outp""pedi""reco""moth""fath"                  ,""     ,"    dp     ",out_ord_moth   ,0},
{"outp""pedi""vari""all"                         ,""     ,"    d      ",out_all        ,0},
{"outp""resc""file"                              ,"s"    ,"      al   ",out_resc_fil   ,0},
{"outp""scor""file"                              ,"s"    ,"      a    ",out_scor_fil   ,0},
{"outp""scor""mc"  "iter"                        ,"i"    ,"      a    ",out_scor_iter  ,0},
{"outp""seed""file"                              ,"s"    ,"    d alsi ",out_seed_fil   ,0},
{"outp""spac""em"  "iter"                        ,"i"    ,"ugbm       ",out_space      ,0},
{"outp""stat""corr"                              ,""     ,"   m       ",out_corr       ,0},
{"outp""stat""cova"                              ,""     ,"  bm       ",out_covar      ,0},
{"outp""trai""seed""only"                        ,""     ,"    d      ",out_trait_seeds,0},
{"samp""by"  "scan"                              ,""     ,"      als  ",sampl_by_scan  ,0},
{"samp""by"  "step"                              ,""     ,"      als  ",sampl_by_step  ,0},
{"sele""chro""mark"                              ,"ii"   ,"      als  ",sim_cmark      ,0},
{"sele""chro""mark""trai"                        ,"iii"  ,"      als  ",sim_cmark_trait,0},
{"sele""mark"                                    ,"i"    ,"      als  ",sim_mark       ,0},
{"sele""mark""trai"                              ,"ii"   ,"      als  ",sim_mark_trait ,0},
{"sele""trai"                                    ,"i"    ,"ugbm       ",sel_trait      ,0},
{"sele""trai""effe"                              ,"ik"   ,"ugbm       ",sel_tr_eff     ,0},
{"sele""unli""trai"                              ,"k"    ,"      a    ",sim_unlinked   ,0},
{"set" "bree""conv"                              ,"r"    ,"ugbm       ",set_br_conv    ,0},
{"set" "chro""locu""wind"                        ,"ii"   ,"         i ",set_cloc_wind  ,0},
{"set" "chro""mark""data"                        ,"iia"  ,"      als  ",set_cmark_dat  ,0},
{"set" "chro""mark""freq"                        ,"ikx"  ,"    d als  ",set_cmark_frq  ,0},
{"set" "comp""prob""game"                        ,"ig"   ,"      a  i ",set_cmp_prbgam ,0},
{"set" "eige"                                    ,"x"    ," gbm       ",set_eigen      ,0},
{"set" "l-sa""prob"                              ,"r"    ,"      als  ",set_lsmpl_prob ,0},
{"set" "locu""wind"                              ,"i"    ,"         i ",set_loc_wind   ,0},
{"set" "mark""data"                              ,"ia"   ,"      als  ",set_mark_dat   ,0},
{"set" "mark""freq"                              ,"kx"   ,"    d als  ",set_mark_frq   ,0},
{"set" "mark""seed"                              ,"hh"   ,"    d      ",set_mark_seeds ,0},
{"set" "mc"  "iter"                              ,"i"    ,"      alsi ",set_mc_iter    ,0},
{"set" "peel""seq"                               ,"x"    ,""           ,not_impl       ,0},
{"set" "prob""game"                              ,"g"    ,"      a  i ",set_prbgam     ,0},
{"set" "reco""freq"                              ,"x"    ,"          k",set_reco_frq   ,0},
{"set" "resi""corr"                              ,""     ,"    d      ",not_impl       ,0},
{"set" "resi""cova"                              ,""     ,"    d      ",not_impl       ,0},
{"set" "samp""seed"                              ,"hh"   ,"      alsi ",set_sampl_seeds,0},
{"set" "scor""iter"                              ,"i"    ,"      als  ",set_scor_iter  ,0},
{"set" "trai""addi""vari"                        ,"kr"   ,"    d   s  ",set_addi_var   ,0},
{"set" "trai""data""geno"                        ,""     ,"      als  ",set_trait_geno, 0},
{"set" "trai""data""phen"                        ,""     ,"      als  ",set_trait_pheno,0},
{"set" "trai""freq"                              ,"kx"   ,"    d als  ",set_trait_frq  ,0},
{"set" "trai""geno""mean"                        ,"kx"   ,"    d   s  ",set_geno_mean  ,0},
{"set" "trai""resi""vari"                        ,"kr"   ,"    d   s  ",set_resi_var   ,0},
{"set" "trai""seed"                              ,"hh"   ,"    d      ",set_trait_seeds,0},
{"simu""chro""mark"                              ,"ii"   ,"    d    i ",sim_cmark      ,0},
{"simu""chro""mark""trai"                        ,"iii"  ,"    d    i ",sim_cmark_trait,0},
{"simu""mark"                                    ,"i"    ,"    d    i ",sim_mark       ,0},
{"simu""mark""trai"                              ,"ii"   ,"    d    i ",sim_mark_trait ,0},
{"simu""unli""trai"                              ,"k"    ,"    d    i ",sim_unlinked   ,0},
{"star""addi""corr"                              ,"x"    ,"  bm       ",start_add_cor  ,0},
{"star""addi""cova"                              ,"x"    ,"  bm       ",start_add_cov  ,0},
{"star""addi""vari"                              ,"x"    ,"ugbm       ",start_add_var  ,0},
{"star""resi""corr"                              ,"x"    ,"  bm       ",start_res_cor  ,0},
{"star""resi""cova"                              ,"x"    ,"  bm       ",start_res_cov  ,0},
{"star""resi""vari"                              ,"x"    ,"ugbm       ",start_res_var  ,0},
{"star""trai""effe"                              ,"iix"  ,"ugbm       ",start_effect   ,0},
{"star""trai""mean"                              ,"ir"   ,"ugbm       ",start_mean     ,0},
{"titl"                                          ,"s"    ,"           ",not_impl       ,0},
{"use" "full""em"                                ,""     ,"   m       ",use_full_em    ,0},
{"use" "l-sa""for" "setu"                        ,""     ,"      als  ",use_lsmpl_setup,0},
{"use" "part""em"                                ,""     ,"   m       ",use_part_em    ,0},
{"use" "sequ""impu""for" "setu"                  ,""     ,"      als  ",use_sqimp_setup,0},
{""                                              ,""     ,""           ,not_impl       ,0}};


/*  descriptions of input statements, SAME ORDER and SAME NUMBER        */
/*  as the statement prototypes above                                   */

static char * input_statement[] = {
        "allow component size  I"                                         ,
        "allow observed individuals  I"                                   ,
        "allow pedigree size  I"                                          ,
        "assign gender"                                                   ,
        "assume all observed"                                             ,
        "check eigenvalues"                                               ,
        "check eigenvalue computation"                                    ,
        "check ginverse"                                                  ,
        "check gmatrix"                                                   ,
        "check progress MC iterations  I"                                 ,
        "check trace"                                                     ,
        "compute component  J  inbreeding coefficients  N1 N2 ..."        ,
        "compute component  J  kinship coefficients  N1A N1B  N2A N2B ...",
        "compute component  J  two-locus inbreeding coefficients  N1 ..." ,
        "compute eigenvalues"                                             ,
        "compute inbreeding coefficients  N1 N2 ..."                      ,
        "compute kinship coefficients  N1A N1B  N2A N2B ..."              ,
        "compute two-locus inbreeding coefficients  N1 ..."               ,
        "fit additive correlations  K12 ..."                              ,
        "fit additive covariances  K12 ..."                               ,
        "fit environmental model"                                         ,
        "fit residual correlations  K12 ..."                              ,
        "fit residual covariances  K12 ..."                               ,
        "ignore gender"                                                   ,
        "input eigenvalue file  S"                                        ,
        "input marker data file  S"                                       ,
        "input pedigree file  S"                                          ,
        "input pedigree record father mother"                             ,
        "input pedigree record gender absent"                             ,
        "input pedigree record gender present"                            ,
        "input pedigree record integers  J"                               ,
        "input pedigree record integers  J  reals  K"                     ,
        "input pedigree record names  I"                                  ,
        "input pedigree record names  I  integers  J"                     ,
        "input pedigree record names  I  integers  J  reals  K"           ,
        "input pedigree record names  I  reals  K"                        ,
        "input pedigree record mother father"                             ,
        "input pedigree record observed absent"                           ,
        "input pedigree record observed present"                          ,
        "input pedigree size  I"                                          ,
        "input peeling sequence file  S"                                  ,
        "input rescue file  S"                                            ,
        "input seed file  S"                                              ,
        "label chromosome  I  marker  J  S"                               ,
        "label marker  J  S"                                              ,
        "label trait  I  S"                                               ,
        "limit breeding iterations  I"                                    ,
        "limit EM iterations  I"                                          ,
        "map chromosome  I  gender F  marker distances  X1 ..."           ,
        "map chromosome  I  gender F  marker positions  X1 ..."           ,
        "map chromosome  I  gender F  marker "
                                        "recombination fractions  X1 ..." ,
        "map chromosome  I  gender F  trait  J  "
                                             "external distances  X1 ..." ,
        "map chromosome  I  gender F  trait  J  "
                               "external recombination fractions  X1 ..." ,
        "map chromosome  I  gender F  trait  J  "
                                        "interval proproportions  X1 ..." ,
        "map chromosome  I  gender F  traits  J1 ...  markers  K1 ...  "
                                                      "distances  X1 ..." ,
        "map chromosome  I  gender F  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map chromosome  I  gender M  marker distances  X1 ..."           ,
        "map chromosome  I  gender M  marker positions  X1 ..."           ,
        "map chromosome  I  gender M  marker "
                                        "recombination fractions  X1 ..." ,
        "map chromosome  I  gender M  trait  J  "
                                             "external distances  X1 ..." ,
        "map chromosome  I  gender M  trait  J  "
                               "external recombination fractions  X1 ..." ,
        "map chromosome  I  gender M  trait  J  "
                                        "interval proproportions  X1 ..." ,
        "map chromosome  I  gender M  traits  J1 ...  markers  K1 ...  "
                                                      "distances  X1 ..." ,
        "map chromosome  I  gender M  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map chromosome  I  marker distances  X1 ..."                     ,
        "map chromosome  I  marker positions  X1 ..."                     ,
        "map chromosome  I  marker recombination fractions  X1 ..."       ,
        "map chromosome  I  trait  J  external distances  X1 ..."         ,
        "map chromosome  I  trait  J  "
                               "external recombination fractions  X1 ..." ,
        "map chromosome  I  trait  J  interval proportions  X1 ..."       ,
        "map chromosome  I  traits  J1 ...  markers  K1 ...  "
                                                      "distances  X1 ..." ,
        "map chromosome  I  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map gender F  marker distances  X1 ..."                          ,
        "map gender F  marker positions  X1 ..."                          ,
        "map gender F  marker recombination fractions  X1 ..."            ,
        "map gender F  trait  J  external distances  X1 ..."              ,
        "map gender F  trait  J  external recombination fractions  X1 ...",
        "map gender F  trait  J  interval proportions  X1 ..."            ,
        "map gender F  traits  J1 ...  markers  K1 ...  distances  X1 ...",
        "map gender F  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map gender M  marker distances  X1 ..."                          ,
        "map gender M  marker positions  X1 ..."                          ,
        "map gender M  marker recombination fractions  X1 ..."            ,
        "map gender M  trait  J  external distances  X1 ..."              ,
        "map gender M  trait  J  external recombination fractions  X1 ...",
        "map gender M  trait  J  interval proportions  X1 ..."            ,
        "map gender M  traits  J1 ...  markers  K1 ...  distances  X1 ...",
        "map gender M  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map marker distances  X1 ..."                                    ,
        "map marker positions  X1 ..."                                    ,
        "map marker recombination fractions  X1 ..."                      ,
        "map trait  J  external distances  X1 ..."                        ,
        "map trait  J  external recombination fractions  X1 ..."          ,
        "map trait  J  interval proportions  X1 ..."                      ,
        "map traits  J1 ...  markers  K1 ...  distances  X1 ..."          ,
        "map traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "output marker seeds only"                                        ,
        "output pedigree chronological"                                   ,
        "output pedigree file  S"                                         ,
        "output pedigree record father mother"                            ,
        "output pedigree record mother father"                            ,
        "output pedigree variables all"                                   ,
        "output rescue file  S"                                           ,
        "output scores file  S"                                           ,
        "output scores MC iterations  I"                                  ,
        "output seed file  S"                                             ,
        "output spacing  I  EM iterations"                                ,
        "output statistics correlations"                                  ,
        "output statistics covariances"                                   ,
        "output trait seeds only"                                         ,
        "sample by scan"                                                  ,
        "sample by step"                                                  ,
        "select chromosome  I  markers  J"                                ,
        "select chromosome  I  markers  J  traits  K"                     ,
        "select markers  J"                                               ,
        "select markers  J  traits  K"                                    ,
        "select trait  I"                                                 ,
        "select trait  I  effects  J1 ..."                                ,
        "select unlinked traits  K1 ..."                                  ,
        "set breeding convergence  X"                                     ,
        "set chromosome  I  locus window  J"                              ,
        "set chromosome  I  markers  J  data  N1  M1 M2 ..."              ,
        "set chromosome  I  markers  K1 ...  frequencies  X1 ... "        ,
        "set component  J  proband gametes  N1 K1  N2 K2 ..."             ,
        "set eigenvalues  X1 ..."                                         ,
        "set L-sampler probability  X"                                    ,
        "set locus window  J"                                             ,
        "set markers  J  data  N1  M1 M2 ..."                             ,
        "set markers  K1 ...  frequencies  X1 ..."                        ,
        "set marker seeds  H1  H2"                                        ,
        "set MC iterations  I"                                            ,
        "set peeling sequence  X1 ..."                                    ,
        "set proband gametes  N1 K1  N2 K2 ..."                           ,
        "set recombination frequencies X1 ..."                            ,
        "set residual correlations  X12 ..."                              ,
        "set residual covariances  X12 ..."                               ,
        "set scoring iterations I"                                        ,
        "set sampler seeds  H1  H2"                                       ,
        "set traits  J1 ...  additive variances  X1 ..."                  ,
        "set trait data genotype"                                         ,
        "set trait data phenotype"                                        ,
        "set traits  J1 ...  frequencies  X1 ..."                         ,
        "set traits  J1 ...  genotype means  X1 ..."                      ,
        "set traits  J1 ... residual variances  X1 ..."                   ,
        "set trait seeds  H1  H2"                                         ,
        "simulate chromosome  I  markers  J"                              ,
        "simulate chromosome  I  markers  J  traits  K"                   ,
        "simulate markers  J"                                             ,
        "simulate markers  J  traits  K"                                  ,
        "simulate unlinked traits  K1 ..."                                ,
        "start additive correlations  X12 ..."                            ,
        "start additive covariances  X12 ..."                             ,
        "start additive variances  X1 ..."                                ,
        "start residual correlations  X12 ..."                            ,
        "start residual covariances  X12 ..."                             ,
        "start residual variances  X1 ..."                                ,
        "start trait  I  effect  J  X1 ..."                               ,
        "start trait  I  mean  X"                                         ,
        "title  S"                                                        ,
        "use full EM"                                                     ,
        "use l-sampler for setup"                                         ,
        "use partitioned EM"                                              ,
        "use sequential imputation for setup"                             ,
        ""                                                              } ;

/*  these words may begin a new input statement, thus delimiting the    */
/*  previous statement.                                                 */

static char * verb[] = {
             "allo",
             "assi",
             "assu",
             "chec",
             "comp",
             "fit",
             "igno",
             "inpu",
             "labe",
             "limi",
             "map",
             "outp",
             "samp",
             "sele",
             "set",
             "simu",
             "star",
             "titl",
             "use",
             "****",
             ""     };

#endif  /*PARSEDEFS_H*/

