
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       parsepars.h:  header file for the Pars/ library               */
/*                     -  of file scope                                */ 
/*                     -  included by Pars/ functions                  */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  PARSEPARS_H            
#define  PARSEPARS_H

#define MA1ST   1                       /*Mother comes 1st in record    */
#define PA1ST   2                       /*Father comes 1st in record    */

#define SIGSQ(r, c)  SigSq[(r) * (r - 1) /2 + c - 1]
#define TAUSQ(r, c)  TauSq[(r) * (r - 1) /2 + c - 1]
#define SIG_MODL(r, c)  SigModel[(r) * (r - 1) /2 + c - 1]
#define TAU_MODL(r, c)  TauModel[(r) * (r - 1) /2 + c - 1]

static FILTAB_PTR  pfloop, pft;
static FILE * pfile;

 
/***    limits overriding variables                                   ***/
 
static int    allw_pedsiz,  allw_cmpsiz,   allw_obsiz;
 
/***    record variables                                              ***/

static int parentOrdIn  = UNDEF;
static int parentOrdOut = UNDEF;

/***    the following macro constants are used by proc_pars() and     ***/
/***    proc_drop()                                                   ***/

#define NCH  0      /* no chromosome     */
#define CH   1      /* chromosome        */
#define NCMP 0      /* no component      */
#define CMP  1      /* component         */

#define NO_CH_NO -1 /* chrom not needed  */

#define NT 0        /* no traits         */
#define T  1        /* traits            */

#define R  0        /* recomb fraction   */
#define D  1        /* distance          */
#define P  2        /* position          */
#define I  3        /* intrval proportion*/ 

#define MD  0       /* marker data       */
#define TD  1       /* trait data        */

#define AV  0       /* additive variance */
#define RV  1       /* residual variance */

#define NIL -1      /* ignore            */


/***    the following data structures are used by proc_pars() and     ***/
/***    proc_kin() for PedComp/kinship computing requests             ***/

struct kinship_node {                    /*Kinship_node:  the no. of    */
    int component;                       /*probands for this component  */
    int no_nams;                         /*and a pointer to the list    */
    char * p_nams;                       /*of names                     */
    struct kinship_node * p_next;
                    };
typedef struct kinship_node KINS;
typedef KINS * KINS_PTR;
typedef KINS * INBR_PTR;
typedef KINS * INBR2_PTR;
static KINS_PTR  first_kins_node = NULL;
static INBR_PTR  first_inbreed_node = NULL;
static INBR2_PTR first_2loc_node = NULL;

 
/***    the following data structures are used by proc_pars() and     ***/
/***    proc_drop() for genedrop simulation specifications            ***/

struct unlnkd_node {                    /*Unlnkd_node:  unlinked traits */
    int id;
    struct unlnkd_node * p_next;
                   };
typedef struct unlnkd_node UNLNKD;
typedef UNLNKD * UNLNKD_PTR;
static UNLNKD_PTR first_unlnkd_node = NULL;


struct sim_node {                       /*Sim_node:  no. of markers and */
    int chrom_no;                       /*no. of linked traits simulated*/
    int no_mark;                        /*on this chromosome            */
    int no_traits;
    struct sim_node * p_next;
                 };
typedef struct sim_node SIM;
typedef SIM * SIM_PTR;
static SIM_PTR first_sim_node = NULL;


struct map_node {                       /*Map_node:  no. of markers or  */
    int chrom_no;                       /*traits on this chromosome and */
    int no_loci;                        /*pointer to list of distances  */
    struct dist_list * p_dist;          /*for these loci                */
    struct map_node * p_next;           /*3 lists:  female, male and    */
                 };                     /*sex-averaged                  */
typedef struct map_node MAP;
typedef MAP * MAP_PTR;
static MAP_PTR first_mark_map_node[3] = {NULL, NULL, NULL};
static MAP_PTR first_trait_map_node[3] = {NULL, NULL, NULL};

struct dist_list {                      /*Dist_node:  distance of marker*/
    int id;                             /*or trait from preceding marker*/
    int mark_id;                        /*on this chromosome            */
    double dist;     
    double prob;
                  };
typedef struct dist_list DIST;
typedef DIST * DIST_PTR;


struct vmap_node {                      /*Vmap_node:  for this          */
    int chrom_no;                       /*chromosome and trait with     */
    int id;                             /*varying positions, the no. of */
    int no_prop;                        /*proportional distances between*/ 
    double * p_prop;                    /*markers and pointer to the    */
    int no_xtrn;                        /*list;  no. of external dists, */
    double * p_dist;                    /*and pointers to cM's and      */
    double * p_rf;                      /*recombination fractions       */
    struct vmap_node * p_next;          /*3 lists:  female, male and    */
                 };                     /*sex-averaged                  */
typedef struct vmap_node VMAP;
typedef VMAP * VMAP_PTR;
static VMAP_PTR first_trait_vmap_node[3] = {NULL, NULL, NULL};


struct locus_node {                     /*Locus_node:  the number of    */
    int chrom_no;                       /*alleles for this loci and a   */
    int id;                             /*pointer to the allele         */
    int no_alleles;                     /*frequency_node                */
    struct freq_node * p_freq_node;
    struct locus_node * p_next;
                    }; 
typedef struct locus_node LOCUS;
typedef LOCUS * LOCUS_PTR;
static LOCUS_PTR first_mark_locus_node = NULL;
static LOCUS_PTR first_trait_locus_node = NULL;

struct freq_node {                      /*Freq_node:  the number of loci*/
    int no_loci;                        /*with these allele frequencies */
    double * p_freq;                    /*and pointer to list of freqs  */
                 };
typedef struct freq_node FREQ;
typedef FREQ * FREQ_PTR;


struct means_node {                     /*Means_node: the genotype      */
    int id;                             /*means (3) for this trait      */
    double mean[3];
    struct means_node * p_next;
                   };
typedef struct means_node MEANS;
typedef MEANS * MEANS_PTR;
static MEANS_PTR first_means_node = NULL;


struct vari_node {                      /*Vari_node:  additive or       */
    int id;                             /*residual variance for the     */
    double vari;                        /*trait                         */
    struct vari_node * p_next;
                  };
typedef struct vari_node VARI;
typedef VARI * VARI_PTR;
static VARI_PTR first_addi_node = NULL;
static VARI_PTR first_resi_node = NULL;


/***    the following data structures are used by proc_pars() and     ***/
/***    proc_auto() for Autozyg/ specifications                       ***/

struct markdat_node {                   /*Markdat_node:  for ea. chrom, */
    int chrom_no;                       /*the number of observed        */
    int no_obs;                         /*individuals                   */
    int no_marks;                       /*the number of markers         */
    char * p_obs;                       /*a pointer to the list of names*/
    int * p_alls;                       /*a pointer to the obs alleles  */
    struct markdat_node * p_next;    
                 };                 
typedef struct markdat_node MARKDAT;
typedef MARKDAT * MARKDAT_PTR;
static MARKDAT_PTR first_markdat_node = NULL;

struct probgam_node {                    /*Probgam_node:  the no. of    */
    int component;                       /*proband gametes for this     */
    int no_probs;                        /*component and a pointer to   */
    struct probgam_list *  p_pglist;     /*list of names and gametes    */
    struct probgam_node * p_next;
                 };
typedef struct probgam_node PROBGAM;
typedef PROBGAM * PROBGAM_PTR;
static PROBGAM_PTR first_probgam_node = NULL;

struct probgam_list {                    /*Probgam_list:  the list of   */
    char nam[MAXNAMLEN + 1];             /*individuals and gametes for  */
    int gamete;                          /*the component                */
                  };
typedef struct probgam_list PROBANDS;
typedef PROBANDS * PROBLIST_PTR;

struct locwind_node {                   /*Locwind_node:  the locus      */
    int chrom_no;                       /*window size for this chrom    */
    int window;
    struct locwind_node * p_next;
                  };
typedef struct locwind_node LOCWND;
typedef LOCWND * LOCWND_PTR;
static LOCWND_PTR first_locwind_node = NULL;


/***  the following variables and tables, used by proc_pars(),        ***/
/***  proc_poly() and check_pars(), contain PolyEM/ modelling         ***/
/***  specifications                                                  ***/

static int    add_cor_flag,             /*Starting values are correla-  */
              res_cor_flag;             /*tions, not covariances        */

static int    naddvar, naddcov,         /*No. starting values input     */
              nresvar, nrescov;         /*for variances and covariances */

static int    nfitadd, nfitres;         /*No. fit flags for covariances */

static int    seltrff[FMAX_REAL][FMAX_FEF];    /*Selection/nonselection */   
                                        /*of traits & fixed effects.    */
                                        /*Same as ffdex[][], but not    */
                                        /*condensed for traits selected */
                                   
static int    nstartff[FMAX_REAL][FMAX_FEF];   /*No. starting values for*/
                                        /*levels for the fixed effect   */
                                        /*(including mean) for the trait*/

static double startff[FMAX_REAL][FMAX_FEF][LEVELS];    /*Starting values*/
                                        /*for fixed effect levels/means */ 

static int    selff[FMAX_FEF];          /*Selection/nonselection of     */
                                        /*fixed effects (incl mean)     */

static int    eigen_set, eigen_compute; /*Eigenvalue options            */

static int    n_eigen;                  /*No. eigenvalues given         */


#endif  /*PARSEPARS_H*/

