          
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       pedchk.h:  data structures used by pedchk functions           */
/*                  also function prototypes for Pedchk/ functions     */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/


#ifndef PEDCHK_H
#define PEDCHK_H

#include "incs.h"

                                        /*For writing:                  */
GLOBAL int widPNam;                     /*max length of names           */
GLOBAL int minPInt[FMAX_INT-3],         /*smallest and                  */
           maxPInt[FMAX_INT-3],         /*largest integer               */
           widPInt[FMAX_INT-3];         /*no. characters                */

GLOBAL int nInd;                        /* no. individuals found in     */
                                        /* pedigree                     */
GLOBAL int nObs;                        /* no. observed individuals     */
                                        /* in pedigree                  */

                        /*indKeys:  key table for individuals           */
struct indKeys {                        
    char nam  [MAXNAMLEN+1];            /*input name                    */      
    int ix;                             /*input record no. (0, 1, ...)  */
    int chron;                          /*chronological no. (1, 2, ...) */
    int founder;                        /*founder?                      */
    int ixMa;                           /*record no. for mother         */
    int ixPa;                           /*record no. for father         */
               };
typedef struct indKeys KEYS;
typedef KEYS * KEYS_PTRS;
typedef KEYS_PTRS * KEYS_PP;
GLOBAL KEYS_PTRS pFirstKeyTab; 
GLOBAL KEYS_PP ppKeyTab;

                        /*pedDat:  pedigree data input for indivduals   */
struct pedDat {
    char nam  [MAXNAMLEN+1];
    char namMa[MAXNAMLEN+1];
    char namPa[MAXNAMLEN+1];
    int sex;
    int obsrvd;
              };
typedef struct pedDat PEDD;
typedef PEDD * PED_PTR;
GLOBAL PED_PTR pPedDat;

                        /*intDat:  optional integer data in input       */
                        /*         pedigree file for individuals        */
typedef int * PEDI_PTR;
GLOBAL PEDI_PTR pIntDat;

                        /*RSDat:  optional real data from input         */
                        /*           pedigree file for individuals      */
typedef char * PEDRS_PTR;
typedef PEDRS_PTR * PEDRS_PP;
GLOBAL PEDRS_PP ppRSDat;
GLOBAL int maxRSDat;                    /*no. characters allocated      */
GLOBAL int curRSDat;                    /*no. characters used           */


void   allocPed(void),            reallocRSDat(void),   /* ped.c        */
       writChkPed(FILE *),        freeChkPed(void);
int    readPed(FILE *),           checkPed(FILE *),
       assignChkPed(void);
FILE * openChkPed(void);

void   allocKey(void),            storKey(void),        /* key.c        */
       freeKey(void);

int    checkChron(void),          newChron(void);       /* chron.c      */
void   assignChron(void),
       reassignIx(void);

int    checkNam(void);                                  /* nam.c        */

void   findParents(void);                               /* parent.c     */
int    checkParents(void); 
int    unknown (char *);

void   assignGender(void);                              /* gender.c     */
int    checkGender(void);

void   sortChron(void),           sortNam(void),        /* sort.c       */
       sortIx(void);
int    comparNam(const void *, const void *),
       comparIx(const void *, const void *),
       comparChron(const void *, const void *);

int    chk_pedfil(void),          mak_opedfil(void);     

#endif /*PEDCHK_H*/
