
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*        quant.h:  header file for Quant/                             */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef QUANT_H
#define QUANT_H

#include "nghds.h"
#include "cmf.h"

GLOBAL double mu[MAX_FEP];              /*mu[]:  PolyEM vector          */

GLOBAL int  np;                         /*Number actually sampled       */

GLOBAL double **x;                      /* the quantitative data;
                  one set for each trait will be allocated (pedsize + 1)
                  in input routines                                     */
GLOBAL int *qdex, *idex;                /* pedigree to data indexing;
                  will be allocated size (pedsize +1) in input routines */
GLOBAL int *fixdex[FIXEFF];             /* fixed effects indicators;
                  will be allocated size (pedsize +1) in input routines */

typedef struct _Term { int r1; int r2; double value; } Term;

double quarz(double *, double *, Term *, int);
double quizza(double *);
double innerprod(double *, double *, int);
double matprod(double *, double **, int);
double fixeffit(double *, int, int);
void   setfixeff(void);
double invtrc(double *);
double logdet(double *);
double invdet(double *);
double totvar(double *);
int    henderson(Term *);
double invert_mat(double **, int);
void   cheigen (double **, double *, int);
void   out_fixed(double *, int);
void   out_bar_one(double *, double *, double *, int, int);
void   out_gene (double *ag, double *a, double *mu_vals, int *g,
                                                double lam, int im);
int    column_test(double *, int, int, int *, int);
int    reduce(double *, double *, int *, int *, int, Ind **);
void   bigcheig(double *, int, int, Ind **, int *);
int    input_data(FILE *, int, int **);
void   free_input_data(void);
double envonly (double *xx);
void   likval (int, double *, double *, double *, double *, double **);
void   lik_big (int, double *, double *, double *, double *, Term *, int);
void   lik_direct (double *, double *, double **);
void   lik_gauss (double *);
void   lik_peel(int *, double *);
void   poly_print_pars (int, int);

double lik_eigen(double **, double *, int, double *);
double comput_lods (double **, double *, int, int, double *);

#endif  /*QUANT_H*/

