/****************************************************************************
 *                                                                          *
 *    Penetrance routines moved from lsample_peel.c: EAT April 2002         *
 *                                                                          *
 ****************************************************************************/

#include "lsample.h"

static double *res, sige, konst;

void  set_qtlpen (double *ym, double ve)
/* sets up static data vector and constants for qtlpen  */
{
   res = ym;
   sige = ve;
   konst = 1.0 / sqrt (PPI * sige);
}

/* Change so that we are multiplying the input vector by the penetrance 
 * (we just zero out the impossible terms in this case)
 * Also change so we get the number of alleles from the locus structure
 * rather than the global n_all, allowing us to use the same routine with
 * both L and M samplers - SCH June 2001 */
void  recessive (double *val, const Ind * id, const struct Locus *locus,
                 double lam)
{
   int   obs, k;
   if (locus->n_all != 2)
      printf ("\n WARNING: trait locus has %d alleles\n", locus->n_all);
   obs = qdex[id->index];
   if (obs > 0)
   {
      k = trtphen[id->index];
      if (k == 4)
      {
         val[0] = 0.0;
         val[1] = 0.0;
         val[2] = 0.0;
      }
      else if (k == 3)
      {
         val[0] = 0.0;
         val[3] = 0.0;
      }
      else if (k == 2)
      {
         val[3] = 0.0;
      }
      else if (k == 1)
      {
         val[1] = 0.0;
         val[2] = 0.0;
         val[3] = 0.0;
      }
      else
         printf ("\n UNRECOGNIZED TRAIT PHENOTYPE of %s is %d",
                 id->self, k);
   }
   return;
}

void  qtlpen (double *val, const Ind * id, const struct Locus *locus,
              double lam)
{
   double *g_mu, z, y;
   int   i, j, k, obs;

   if (CHK_PEEL (TRACE_LEVEL_1))
      printf ("Inserting penetrance probabilities for %s\n", id->self);
   obs = qdex[id->index];
   if (obs > 0)
   {
      g_mu = locus->g_mu;
      y = res[obs];
      k = 0;
      for (i = 0; i < locus->n_all; i++)
         for (j = 0; j <= i; j++)
         {
            z = y - lam * g_mu[k++];
            z = exp (-.5 * z * z / sige) * konst;
            val[i * locus->n_all + j] *= z;  /* EAT bug fixed, 4/2002 */
            if (i != j)
               val[j * locus->n_all + i] *= z;  /* EAT bug fixed. 4/2002 */
         }
   }
}
