
   /*******************************************************************
    * L-Sampler Setup ; code by SCH, Summer 1999                      *
    *                                                                 *
    *   installed as separate subroutine; EAT, March 2000             *
    *******************************************************************/

#include "lsample.h"

void  set_positions (int nloc)
{
   int   i, j, k;
   double zm, zf, rr;
   /* Set up positions for lsampler routines */

   zf = 0.0;
   zm = 0.0;
   j = 1;
   for (i = 0; i <= nloc; i++)
   {
      k = chromo[0][i];
      loci[k].chromosome = j;
      loci[k].malepos = zm;
      loci[k].fempos = zf;
      rr = xecomo[1][0][i];             /* male  map; EAT, 2/00 */
      if (rr == 0.5)
      {
         j++;
         zm = 0.0;
      }
      else
         zm += 100.0 * gendist (rr);    /* Get distance in cM */

      rr = xecomo[0][0][i];             /* female  map; EAT, 2/00 */
      if (rr == 0.5)
         zf = 0.0;                      /* bug fixed EAT March 2002 */
      else
         zf += 100.0 * gendist (rr);    /* Get distance in cM */
   }
   return;
}

void  set_lsmpl (int rs, int nloc, int *nall, double **Qq, int ***y)
{
   double z;
   int   i;

   /* Allocate structures needed for L-Sampler */
   allocate_loci (nloc + 1, nall, Qq);

   /* Set up positions for lsampler routines */
   set_positions (nloc);

   /* In L-Sampler terms, we have nloc+1 markers and no trait loci,
    * as trait loci are loci with general penetrance functions */
   n_markers = nloc + 1;
   n_tloci = 0;
   fputc ('\n', stdout);

   /* Initialize peeling routines, find peeling sequence etc.
    * The first parameter is zero unless you're giving it
    * phase known data when it's a pointer to an int array of
    * size pedsize.  In this case individual i has phase known
    * data if the corresponding array entry is non-zero.
    * The second parameter is a quiet flag - if non-zero then no
    * messages are output *unless* an error is found.
    * The third parameter is the genotype data array.
    */
   locus_peel_init (0, 1, y);
/*      locus_peel_init(0,0,y);                   quiet_flag not set  */
   fflush (stdout);
   peel_trace = 0;                      /* Set reporting level for peeling 
                                           functions */
   /* realise the initial switches if not already read above */
   if (rs == 0)
   {
      set_transmission (0, 0, 0);
      /* Initial sample is for unlinked loci. This is required. */
      fflush (stdout);
      for (i = 0; i <= nloc; i++)
      {
         /* Peel/sample routine.
          * First parameter is double array with residuals for
          *    quantitative trait (not used here).
          * Second parameter is residual variance (not used here).
          * Third parameter is lam - only used in mixt I think).
          * Fourth parameter is sample flag, 0 = peel only, >0 = sample
          * Fifth parameter is pointer to locus structure to be peeled/sampled
          */
         fflush (stdout);
         z = locus_mjrpeel (0, 1.0, 1.0, 1, loci + i);
         fflush (stdout);
         if (z == -DBL_MAX)
         {
            fprintf (stderr, "Zero probability in peeling operation\n");
            exit (-1);
         }
/*****  printf ("log-likelihood (Locus %d) = %-20f\n",i,z);  	*****/
      }
   }

}
