/* functions called by lm_auto, lm_lods, ibddrop for allocating and 
 * freeing arrays  
 *
 *  alloc_y(), alloc_ibd(), alloc_wndw(), alloc_lod(), alloc_loglik()
 *  free_y(), free_ibd(),  free_wndw(), free_lod(),  free_loglik()
 *                                                              MAJ Aug 01  
 *
 * functions called by lm_lange that are different from Autozyg versions
 *  alloc_loglik(), free_loglik()                               EAT Aug 01  
 *
 *  All above moved to and combined in Msample library          EAT Feb 02
 *     Items from alloc_az_prob and free_az_prob moved to 
 *        alloc/free _seg_pen, _loglik, and _ibd as appropriate        
 *     Part of alloc_ld_prob  moved to alloc_loglik -- remainder
 *        renamed alloc_lod; ditto with the free_*
 * 
 *  allocz_y(), renamed alloc_y();  args changed for alloc_y() and free_y()
 *  alloc_wndw(), free_wndw() added for locus window option for ibddrop
 *                                                              MAJ Apr 02
 */

#include "msample.h"

int ***alloc_y (int nl, int np)
{

   int ***y;

   y = malloc (2 * sizeof (int **));
   alloc_chk (WHERE, y);
   y[0] = alloc_genes (nl, np);
   y[1] = alloc_genes (nl, np);

   return y;
}

void  alloc_ibd (int **g, int ***ix, int ***lab, int ***lst, int **np,
                 int ***c, double *****nsc, double **s)
{
/*  int g   [G]
 *  int ix  [G+1][CD+1]
 *  int lab [G+1][PT+1]
 *  int lst [G+1][PT+1]
 *  int npt [G+1]
 *  int c      [nc][nl]            moved here EAT 2/2002
 *  double nsc [NC][NS][NL][PT+1]
 *  double s   [nl]                moved here EAT 2/2002
 *
 *  see table in limdefs.h for no. gametes/codes/patterns
 */

   int   i, j, k;

   *np = malloc ((MxGam + 1) * sizeof (int));
   alloc_chk (WHERE, *np);
   (*np)[0] = 0;

   *g = malloc (MxGam * sizeof (int));
   alloc_chk (WHERE, *g);

   *lab = malloc ((MxGam + 1) * sizeof (int *));
   alloc_chk (WHERE, *lab);
   (*lab)[0] = NULL;

   *ix = malloc ((MxGam + 1) * sizeof (int *));
   alloc_chk (WHERE, *ix);
   (*ix)[0] = NULL;

   *lst = malloc ((MxGam + 1) * sizeof (int *));
   alloc_chk (WHERE, *lst);
   (*lst)[0] = NULL;

   /* the index, list and label arrays are allocated as ragged arrays */

   for (i = 1; i <= MxGam; ++i)
   {
      (*ix)[i] = malloc ((CDgg[i] + 1) * sizeof (int));
      alloc_chk (WHERE, (*ix)[i]);
      (*lst)[i] = malloc ((PTgg[i] + 1) * sizeof (int));
      alloc_chk (WHERE, (*lst)[i]);
      (*lab)[i] = malloc ((PTgg[i] + 1) * sizeof (int));
      alloc_chk (WHERE, (*lab)[i]);
   }
   *nsc = malloc (n_compnts * sizeof (double ***));
   alloc_chk (WHERE, *nsc);

   /* the nscore array is ragged in 2 dimensions */

   for (i = 0; i < n_compnts; ++i)
   {
      (*nsc)[i] = malloc (NScSet[i] * sizeof (double **));
      alloc_chk (WHERE, (*nsc)[i]);
      for (j = 0; j < NScSet[i]; ++j)
      {
         (*nsc)[i][j] = malloc (NLoci * sizeof (double *));
         alloc_chk (WHERE, (*nsc)[i][j]);
         for (k = 0; k < NLoci; ++k)
         {
            (*nsc)[i][j][k] = malloc ((PTgg[NPrGam[i][j]] + 1) *
                                      sizeof (double));
            alloc_chk (WHERE, (*nsc)[i][j][k]);
         }
      }
   }
   ali_nc_nl (c);
   ald_nl (s);
   return;
}

void  alloc_wndw (int **g, double *****nsc, int nw, int nwpt)
{
/* int  g[W]
 * double nscore[NC][NS][NW][NWPT]
 */
   int   i, j, k;

   *g = malloc (LocWndw * sizeof (int));
   alloc_chk (WHERE, *g);

   /* the nscore array is ragged in 2 dimensions */

   *nsc = malloc (n_compnts * sizeof (double ***));
   alloc_chk (WHERE, *nsc);

   for (i = 0; i < n_compnts; ++i)
   {
      (*nsc)[i] = malloc (NScSet[i] * sizeof (double **));
      alloc_chk (WHERE, (*nsc)[i]);
      for (j = 0; j < NScSet[i]; ++j)
      {
         (*nsc)[i][j] = malloc (nw * sizeof (double *));
         alloc_chk (WHERE, (*nsc)[i][j]);
         for (k = 0; k < nw; ++k)
         {
            (*nsc)[i][j][k] = malloc (nwpt * sizeof (double));
            alloc_chk (WHERE, (*nsc)[i][j][k]);
         }
      }
   }
   return;
}

void  alloc_lod (int **sc, int ***lsf, double ***lrat, double ***clik,
                 double ***pr2, double ***pr5, double ***pr6)
{
   ali_nc (sc);
   ali_n_nl (lsf, 3);

   ald_nc_n (lrat, 3);
   ald_nc_n (clik, 3);
   ald_nc_nl (pr2);
   ald_nc_nl (pr5);
   ald_nc_nl (pr6);

   return;
}

void  free_y (int ****py)
{

   if (!*py)
      return;

   free_genes (&(*py)[0]);
   free_genes (&(*py)[1]);

   free (*py);
   *py = NULL;

   return;
}

void  free_ibd (int **g, int ***ix, int ***lab, int ***lst, int **np,
                int ***c, double *****nsc, double **s)
{
   int   i, j, k;

   if (!*g)
      return;

   for (i = 0; i <= MxGam; ++i)
   {
      if (*ix && (*ix)[i])
         free ((*ix)[i]);
      if (*lst && (*lst)[i])
         free ((*lst)[i]);
      if (*lab && (*lab)[i])
         free ((*lab)[i]);
   }
   free (*ix);
   *ix = NULL;
   free (*lst);
   *lst = NULL;
   free (*lab);
   *lab = NULL;
   free (*np);
   *np = NULL;
   free (*g);
   *g = NULL;

   if (*nsc)
   {
      for (i = 0; i < n_compnts; ++i)
         if ((*nsc)[i])
         {
            for (j = 0; j < NScSet[i]; ++j)
               if ((*nsc)[i][j])
               {
                  for (k = 0; k < NLoci; ++k)
                     if ((*nsc)[i][j][k])
                        free ((*nsc)[i][j][k]);
                  free ((*nsc)[i][j]);
               }
            free ((*nsc)[i]);
         }
      free (*nsc);
      *nsc = NULL;
   }
   fre_nc_n ((void *) c);
   fre_n ((void *) s);

   free_cst ();

   return;
}
void  free_lod (int **sc, int ***lsf, double ***lrat,
                double ***clik, double ***pr2,
                double ***pr5, double ***pr6)
{
   fre_n ((void *) sc);
   fre_n_n ((void *) lsf, 3);
   fre_nc_n ((void *) lrat);

   fre_nc_n ((void *) clik);
   fre_nc_n ((void *) pr2);
   fre_nc_n ((void *) pr5);
   fre_nc_n ((void *) pr6);

   return;
}

void  alloc_loglik (int ****rc, double ***pr1, double ***pr2,
                    double ***pr3)
/* int rc     [nc][nl][6]
 * double pr1 [nc][nl]
 * double pr2 [nc][nl]
 * double pr3 [nc][nl]
 */
{

   ali_nc_nl_n (rc, 6);
   ald_nc_nl (pr1);
   ald_nc_nl (pr2);
   ald_nc_nl (pr3);

   return;
}

void  free_loglik (int ****rc, double ***pr1, double ***pr2,
                   double ***pr3)
{
   fre_nc_nl_n ((void *) rc);
   fre_nc_n ((void *) pr1);
   fre_nc_n ((void *) pr2);
   fre_nc_n ((void *) pr3);
}

void  free_wndw (int **g, double *****nsc, int nw, int nwpt)
{
   int   i, j, k;

   if (!*g)
      return;

   free (*g);
   *g = NULL;

   if (*nsc)
   {
      for (i = 0; i < n_compnts; ++i)
         if ((*nsc)[i])
         {
            for (j = 0; j < NScSet[i]; ++j)
               if ((*nsc)[i][j])
               {
                  for (k = 0; k < nw; ++k)
                     if ((*nsc)[i][j][k])
                        free ((*nsc)[i][j][k]);
                  free ((*nsc)[i][j]);
               }
            free ((*nsc)[i]);
         }
      free (*nsc);
      *nsc = NULL;
   }
   return;
}
