/* new_pen() replaced by seg_pen()  06/26/01  */

#include "msample.h"

#define CHILL { *err= -1; free(alleles[0]); free(group);\
                  free(obslist); return 0.0; }
                                     /* impossible state */

#define CNOBS { *err= -1; free(group);\
                  free(obslist); return 1.0; }
                                     /* no genes observed */
                    /* BUG corrected 12/31/98 (EAT) -- alleles[0]
                       is NOT allocated in this case--both the allocs and
                       frees in this file need appropriate non-NULL checks */

/* new version of seg-pen, using genes from autozyg programs
   instead of re-setting them each time -- cf old seg_pen.c
   (EAT, June 1997)  */

/*  Modified July 1998, to do computation by component  (EAT) */

double addlog2 (double x1, double x2)
{
   double yy, r;

   yy = .5 * (x1 + x2);
   r = (log (cosh (x1 - yy)) + yy + log (2.0));
   return r;
}

double seg_pen2 (int c, int ***y, double *freq, int locus, int ngenes, int *err)
{
   int  *genes[2];

   genes[0] = gg0[locus];
   genes[1] = gg1[locus];

   return (new_pen (c, genes, y, freq, locus, ngenes, err));
}

double new_pen (int c, int *genes[2], int ***y, double *freq, int locus, 
                int ngenes, int *err)
{
   int   i, ii, j, k, k1, s, t, a1, a2, g1, g2, o1, o2;
   int  *group, *code, *alleles[2], fg[2];
   int   nobsa, *obslist, *groupflag[2];
   Ind  *cpind;
   Indlist *iptr;
   double l, ll[2];

   if (!(obslist = calloc (pedsize, sizeof (int))))
            exit (1);
   *err = 0;
   nobsa = 0;
   /* note we will now compute by component
      so this only has to be size max_cpsize
      (or size of specific component, if we
      knew what that was!)  EAT July 1998 
      note also Nghd components are labelled 1 to nc */
   if (!(group = calloc (ngenes * 2, sizeof (int))))
            exit (1);
   code = group + ngenes;

   /* Group together genes which occur in observed individuals */

   j = 0;
   iptr = cmplist[c + 1].people;
   while (iptr->next != NULL)
   {
      iptr = iptr->next;
      cpind = iptr->current;
      i = cpind->index;
      if (y[0][locus][i])
      {
         obslist[nobsa++] = i;
         a1 = genes[0][i];
         a2 = genes[1][i];
         g1 = group[a1 - 1];
         g2 = group[a2 - 1];
         if (g1 && g2)
         {
            for (k = 0; k < ngenes; k++)
               if (group[k] == g2)
                  group[k] = g1;
         }
         else if (g1)
            group[a2 - 1] = g1;
         else if (g2)
            group[a1 - 1] = g2;
         else
            group[a1 - 1] = group[a2 - 1] = ++j;
      }
   }
   if (nobsa == 0)
      CNOBS;

   /* Recode groups */
   for (i = j = 0; i < ngenes; i++)
      if ((s = group[i]))
      {
         if (!code[s - 1])
            code[s - 1] = ++j;
         group[i] = code[s - 1];
      }

   /* alleles[0][i] and alleles [1][i] will contain the 2 possible allele
    * types for founder gene i */

   if (!(alleles[0] = calloc (ngenes * 2 + j * 2, sizeof (int))))
            exit (1);
   alleles[1] = alleles[0] + ngenes;
   groupflag[0] = alleles[1] + ngenes;
   groupflag[1] = groupflag[0] + j;

   for (i = 0; i < ngenes; i++)
      code[i] = 0;

   /* Sort out possible allele allocations for groups */
   while (nobsa)
   {
      for (ii = 0; ii < nobsa; ii++)
      {
         i = obslist[ii];
         o1 = y[0][locus][i];
         o2 = y[1][locus][i];
         a1 = genes[0][i] - 1;
         a2 = genes[1][i] - 1;

         g1 = group[a1];

         if (!(code[a1] || code[a2]))
         {
            /* Check if another gene in the group is present */

            for (k = 0; k < ngenes; k++)
               if (group[k] == g1 && code[k])
                  break;

            /* If so then skip this observation 'till later */
            if (k < ngenes)
               continue;

            /* First genes in group */
            code[a1] = code[a2] = 1;
            alleles[0][a1] = o1;
            alleles[0][a2] = o2;
            if (a1 != a2)
            {
               if (o1 != o2)
               {
                  alleles[1][a1] = o2;
                  alleles[1][a2] = o1;
               }
               else
                  groupflag[1][g1 - 1] = 1;
            }
            /* check IBD individual is not heterozygous
               -- bug corrected 7/98, EAT  */
            else
            {
               if (o1 != o2)
                  CHILL
                     else
                  groupflag[1][g1 - 1] = 1;
            }
         }

         else if (code[a1])
         {
            k = 0;
            if (code[a2])
            {
               if (!groupflag[0][g1 - 1])
               {
                  if ((alleles[0][a1] == o1 && alleles[0][a2] == o2) ||
                      (alleles[0][a1] == o2 && alleles[0][a2] == o1))
                     k++;
                  else
                     groupflag[0][g1 - 1] = 1;
               }
               if (!groupflag[1][g1 - 1])
               {
                  if ((alleles[1][a1] == o1 && alleles[1][a2] == o2) ||
                      (alleles[1][a1] == o2 && alleles[1][a2] == o1))
                     k++;
                  else
                     groupflag[1][g1 - 1] = 1;
               }
               if (!k)
                  CHILL
                  }
                  else
               {
                  for (k1 = t = 0; t < 2; t++)
                     if (!groupflag[t][g1 - 1])
                     {
                        k = 0;
                        if (alleles[t][a1] == o1)
                        {
                           alleles[t][a2] = o2;
                           k++;
                        }
                        else if (alleles[t][a1] == o2)
                        {
                           alleles[t][a2] = o1;
                           k++;
                        }
                        if (!k)
                           groupflag[t][g1 - 1] = 1;
                        else
                           k1 += k;
                     }
                  if (!k1)
                     CHILL
                        code[a2] = 1;
               }
            }
            else
            {
               for (k1 = t = 0; t < 2; t++)
                  if (!groupflag[t][g1 - 1])
                  {
                     k = 0;
                     if (alleles[t][a2] == o1)
                     {
                        alleles[t][a1] = o2;
                        k++;
                     }
                     else if (alleles[t][a2] == o2)
                     {
                        alleles[t][a1] = o1;
                        k++;
                     }
                     if (!k)
                        groupflag[t][g1 - 1] = 1;
                     else
                        k1 += k;
                  }
               if (!k1)
                  CHILL
                     code[a1] = 1;
            }
            obslist[ii] = obslist[--nobsa];
         }
      }

      l = 0.0;
      for (g1 = 1; g1 <= j; g1++)
      {
         ll[0] = ll[1] = 0.0;
         fg[0] = fg[1] = 1;
         for (i = 0; i < ngenes; i++)
            if (group[i] == g1)
            {
               if (!groupflag[0][g1 - 1])
                  ll[0] += log (freq[alleles[0][i]]);
               else
                  fg[0] = 0;
               if (!groupflag[1][g1 - 1])
                  ll[1] += log (freq[alleles[1][i]]);
               else
                  fg[1] = 0;
            }
         if (fg[0] && fg[1])
            l += addlog2 (ll[0], ll[1]);
         else if (fg[0])
            l += ll[0];
         else if (fg[1])
            l += ll[1];
      }

      free (alleles[0]);
      free (group);
      free (obslist);

      return (exp (l));
   }
