
/*************************************************************************
  Routine to realise switches from the prior.
  There can be cases where this might not be compatible with data.
  April 1994, EAT.

  Added, January 1999; 
     routine to set segs for trait locus using a partial specification
  and
    routine to set marker segs and data conditional on those,
  and
    routine to set genotypes of observed individuals
  (EAT)

  Updated to use the linked list of switches; EAT Jan 2001.

***************************************************************************

  trait_segs(), next_marker_segs() and set_genotypes() not presently used
  by MORGAN;  all were used to simulate marker data conditional on trait
  segregations (MAJ).

  trait_segs(), next_marker_segs()  updated from MORGAN_F1 to use Switch 
  lists etc , so can be used by MORGAN 2.3, and MORGAN 2.4  (EAT: 01/2002)
 
  Modify routines to use standard sex-spec maps, and other MORGAN variables
   (EAT 02/2002)

  realise() is modified for sex-spec maps                      (MAJ 04/02)

***************************************************************************/

#include "msample.h"

void  realise (int nl, double ***recf, int ns)
{
   static int first = 1;
   int   i, j, ii, **xx;
   double *rf;

   if (first)                           /* on first entry, initialize meiosis */
   {                                    /* indicators to -1                   */
      for (i = 0; i < pedsize; ++i)
         for (j = 0; j <= nl; ++j)
         {
            xx0[j][i] = -1;
            xx1[j][i] = -1;
         }
      first = 0;
   }

   for (i = 1; i <= ns; ++i)            /* for each meiosis switch, get       */
   {                                    /* gender (maternal or paternal gene) */
      if (meiosis[i].sex == FEM)
      {
         xx = xx0;
         rf = recf[FEM][0];
      }
      else
      {
         xx = xx1;
         rf = recf[MAL][0];
      }

      ii = (meiosis[i].ind).index;      /* get index for this individual      */

      if (rans () < .5)                 /* randomly choose inheritance for    */
         xx[0][ii] = 0;                 /* first locus                        */
      else
         xx[0][ii] = 1;

      for (j = 1; j <= nl; ++j)         /* for subsequent loci, use sex-      */
      {                                 /* specific recomb fracts             */
         if (rans () > rf[j - 1])
            xx[j][ii] = xx[j - 1][ii];
         else
            xx[j][ii] = !(xx[j - 1][ii]);
      }
   }
   return;
}

void  trait_segs (int ns)
/* this assumes that the trait segs to be conditioned on were read in
   as columns 6 and 7 of the original data matrix */
{
   int   i, ii, ss, **xx, k;
   for (i = 0; i < pedsize; ++i)
   {
      xx0[0][i] = -1;
      xx1[0][i] = -1;
   }
   for (i = 1; i <= ns; ++i)
   {
      ii = (meiosis[i].ind).index;
      ss = meiosis[i].sex;
      if (ss == 0)
      {
         xx = xx0;
         k = 7;
      }
      else
      {
         xx = xx1;
         k = 6;
      }
      xx[0][ii] = data[ii][k];
      if (xx[0][ii] == -1)
      {
         if (rans () < 0.5)
            xx[0][ii] = 0;
         else
            xx[0][ii] = 1;
      }
   }
/*
   printf ("\n list of init segs");
   for (i = 0; i < pedsize; ++i)
   {
   printf ("\n %d %s;", nghd[i].index, nghd[i].self);
   printf ("  %2d %2d", xx1[0][i], xx0[0][i]);
   }
   fflush (stdout);
 */
   return;
}
void  next_marker_segs (int ns, int mf, int mt, int ir)
/* from marker mf, at recombination rec, set marker mt    */
{
   int   i, ii, ss, **xx;
   double recm, recf, rec;
   for (i = 0; i < pedsize; ++i)
   {
      xx0[mt][i] = -1;
      xx1[mt][i] = -1;
   }
   recm = xecomo[1][0][ir];
   recf = xecomo[0][0][ir];

   printf ("\n simulating from locus %d to %d at male and female recs %f %f",
           mf, mt, recm, recf);
   for (i = 1; i <= ns; ++i)
   {
      ii = (meiosis[i].ind).index;
      ss = meiosis[i].sex;
      if (ss == 0)
      {
         xx = xx0;
         rec = recf;
      }
      else
      {
         xx = xx1;
         rec = recm;
      }

      if (xx[mf][ii] != -1)
      {
         if (rans () > rec)
            xx[mt][ii] = xx[mf][ii];
         else
            xx[mt][ii] = 1 - xx[mf][ii];
      }
   }
   return;
}
void  set_genotypes (int nfou, int loc, double *freq, int *gd1, int *gd2)
{
   int  *allotype, k, kk, a1;               /* indexing of types of observed genes */
   double qf, ff;

   allotype = (int *) calloc ((size_t) (2 * nfou + 1), sizeof (int));
   for (k = 1; k <= 2 * nfou; ++k)
      allotype[k] = 0;

   /* trait locus, special case */
   if (loc == 0)
   {
      for (k = 1; k <= 2 * nfou; ++k)
         allotype[k] = 1;
      allotype[2] = 2;
      for (kk = 1; kk <= NObs; ++kk)
      {
         k = ObsIx[kk - 1];
         if (gd2[kk] > 0)
            gg0[loc][k] = allotype[gg0[loc][k]];
         else
            gg0[loc][k] = 0;
         if (gd1[kk] > 0)
            gg1[loc][k] = allotype[gg1[loc][k]];
         else
            gg1[loc][k] = 0;
      }
   }
   /*  marker locus  */
   else
   {
      for (kk = 1; kk <= NObs; ++kk)
      {
         k = ObsIx[kk - 1];
         if (allotype[gg0[loc][k]] == 0)
         {
            qf = rans ();
            a1 = 0;
            ff = 0.0;
            while ((qf > ff) && (a1 < NAll[loc]))
            {
               ++a1;
               ff += freq[a1];
            }
            allotype[gg0[loc][k]] = a1;
         }
         if (allotype[gg1[loc][k]] == 0)
         {
            qf = rans ();
            a1 = 0;
            ff = 0.0;
            while ((qf > ff) && (a1 < NAll[loc]))
            {
               ++a1;
               ff += freq[a1];
            }
            allotype[gg1[loc][k]] = a1;
         }
      }
      for (kk = 1; kk <= NObs; ++kk)
      {
         k = ObsIx[kk - 1];
         gg0[loc][k] = allotype[gg0[loc][k]] * gd2[k];
         gg1[loc][k] = allotype[gg1[loc][k]] * gd1[k];
      }
   }
   free (allotype);
   return;
}
