/****************************************************************************/
/*                                                                          */
/* Traits library.                                           EAT March 02   */
/* routines  set_markdat : moved to Traits library March 02                 */
/* plus new routines:                                                       */
/* set_markmap  -- resets and realloc Marek, Qq and NAll                    */
/* reset_traitmap  -- resets Loc0 and Rec0   for lm_auto                    */
/* unset_recval    -- recovers original input recombs and proportions       */
/* reset_recval    -- resets RecVal and IntVal for lm_lods and other progs  */
/*                                                                          */
/****************************************************************************/

#include "traits.h"                     /* includes msample.h and others included therein */

void  unset_recval (double **ipm, double **ipf, double **rem, double **ref)
{
   int   n_ext, n_int, i;

   n_ext = NTExtPos;
   n_int = NTIntPos;

   *ipm = malloc (n_int * sizeof (double));
   alloc_chk (WHERE, *ipm);
   *ipf = malloc (n_int * sizeof (double));
   alloc_chk (WHERE, *ipf);
   *rem = malloc (n_ext * sizeof (double));
   alloc_chk (WHERE, *rem);
   *ref = malloc (n_ext * sizeof (double));
   alloc_chk (WHERE, *ref);

   for (i = n_ext; i > 0; --i)
   {
      (*ref)[i - 1] = RecVal[0][n_ext + 1 - i];
      (*rem)[i - 1] = RecVal[1][n_ext + 1 - i];
   }
   for (i = n_ext + 1; i <= n_ext + n_int; ++i)
   {
      (*ipf)[i - n_ext - 1] = gendist (RecVal[0][i]) / gendist (Marek[0][1]);
      (*ipm)[i - n_ext - 1] = gendist (RecVal[1][i]) / gendist (Marek[1][1]);
   }
   return;
}

void  reset_recval (int nmarkers,
                    double **ipm, double **ipf, double **rem, double **ref)
{
   int   n_ext, n_int, *na, i, j, k, sx;
   double *mmp;

   n_ext = NTExtPos;
   n_int = NTIntPos;
   NTPos = 2 * n_ext + (nmarkers - 1) * n_int + 1;

   for (i = 0; i <= n_ext; ++i)         /* sets loci for extern positions    */
   {                                    /* plus one each end                 */
      IntVal[i] = 0;
      IntVal[NTPos - i] = nmarkers;
   }

   IntVal[NTPos] = 0;                   /* and loci for interval proportions */
   for (j = 0; j < (nmarkers - 1) * n_int; ++i, ++j)
      IntVal[i] = j / n_int + 1;

   for (sx = FEM; sx <= MAL; ++sx)
   {
      RecVal[sx][0] = .5;               /* set recomb fracts for each end    */
      RecVal[sx][NTPos] = .5;
   }

   j = n_ext;                           /* sets external recomb fracts       */
   k = NTPos - n_ext;
   for (i = 0; i < n_ext; ++i, --j, ++k)
   {
      RecVal[0][j] = (*ref)[i];
      RecVal[0][k] = (*ref)[i];
      RecVal[1][j] = (*rem)[i];
      RecVal[1][k] = (*rem)[i];
   }
   i = n_ext;
   for (k = 1; k < nmarkers; ++k)
      for (j = 0; j < n_int; ++j)
      {
         ++i;
         RecVal[0][i] = recomb ((*ipf)[j] * gendist (Marek[0][k]));
         RecVal[1][i] = recomb ((*ipm)[j] * gendist (Marek[1][k]));
      }

   /* realloc RecVal and IntVal   */
   for (i = FEM; i <= MAL; ++i)
   {
      mmp = realloc (RecVal[i], (NTPos + 1) * sizeof (double));
      alloc_chk (WHERE, mmp);
      RecVal[i] = mmp;
   }

   na = realloc (IntVal, (NTPos + 1) * sizeof (int));
   alloc_chk (WHERE, na);
   IntVal = na;

   /* free other variables        */
   free (*ipf);
   free (*ipm);
   free (*ref);
   free (*rem);

   return;
}

void  set_markmap (int nloc, int nmarkers, int *loc_fl2)
/* resets Marek for the nmarkers to be used in the analysis */
/* resets the allele frequencies and nall, for markers used in the analysis */
/*  Also  if (NVary)                                                        */
/* resets RecVal and Intval for trait positions, relative to marker subset  */
/* This is a really ugly kluge, and should be fixed for future              */
/*  Else:  for lm_auto  resets Loc0, Rec0                                   */
/*   EAT March 2002                                                         */
{
   int   i, j, k, old_marker, new_marker;
   double fem_dist, mal_dist;
   double *mmp, **mmq;
   int   n_ext, n_int, *na;
   double *ipm, *ipf, *rem, *ref;           /* interior propns; exterior recmb */

/* first we recover the external numbers and internal proportions 
   for lod score programs                                           */

   if (NVary)
      unset_recval (&ipm, &ipf, &rem, &ref);

/* or else, in the case of lm_auto, we reset Loc0 and Rec0           */
   else
      reset_traitmap (nloc, nmarkers, loc_fl2);

/* Now start resetting of markers etc                          */
   old_marker = 0;
   new_marker = 0;
   while (old_marker < nloc)
   {
      ++old_marker;
      if (loc_fl2[old_marker] == (new_marker + 1))
      {
         ++new_marker;
         if (new_marker > 1)
         {
            Marek[0][new_marker - 1] = recomb (fem_dist + gendist (Marek[0][old_marker - 1]));
            Marek[1][new_marker - 1] = recomb (mal_dist + gendist (Marek[1][old_marker - 1]));
            Qq[new_marker] = Qq[old_marker];
            NAll[new_marker] = NAll[old_marker];
         }
         fem_dist = 0.0;
         mal_dist = 0.0;
      }
      else
      {
         if (new_marker > 0)
         {
            fem_dist += gendist (Marek[0][old_marker - 1]);
            mal_dist += gendist (Marek[1][old_marker - 1]);
         }
         free (Qq[old_marker]);
      }
   }
   if (new_marker != nmarkers)
      printf ("WARNING: count of markers to be used is inconsistent\n");

   Marek[0][nmarkers] = 0.5;
   Marek[1][nmarkers] = 0.5;

/* Now reset RecVal and IntVal using recovered proportions */

   if (NVary)
      reset_recval (nmarkers, &ipm, &ipf, &rem, &ref);

/* now realloc space for all globals reset   */
/* Marek, Qq and NAll  */
   for (i = FEM; i <= MAL; ++i)
   {
      mmp = realloc (Marek[i], (nmarkers + 1) * sizeof (double));
      alloc_chk (WHERE, mmp);
      Marek[i] = mmp;
   }

   mmq = realloc (Qq, (nmarkers + 1) * sizeof (double *));
   alloc_chk (WHERE, mmq);
   Qq = mmq;
   na = realloc (NAll, (nmarkers + 1) * sizeof (int));
   alloc_chk (WHERE, na);
   NAll = na;

   return;
}

void  reset_traitmap (int nloc, int nmarkers, int *loc_fl2)
/* reset Loc0 and Rec0[sx] for lm_auto       */
/* Need to call this BEFORE resetting Marek  */
/* This can crash if there are NO MARKERS in the new map */
{
   int   i, l0;
   double r0[2], fem_dist, mal_dist;;

   fem_dist = gendist (Rec0[0]);
   mal_dist = gendist (Rec0[1]);

   i = Loc0;
   while ((i > 0) && (loc_fl2[i] < 0))
      --i;
   l0 = i;
   if (l0 > 0)                          /* exists a prev marker in new map */
   {
      for (i = l0; i < Loc0; ++i)
      {
         fem_dist += gendist (Marek[0][i]);
         mal_dist += gendist (Marek[1][i]);
      }
      Rec0[0] = recomb (fem_dist);
      Rec0[1] = recomb (mal_dist);
      Loc0 = loc_fl2[l0];
   }
   else
      /* no previous marker in new map */
   {
      i = Loc0;
      while ((i < nloc) && (loc_fl2[i] < 0))
         ++i;
      l0 = i;
      if (Loc0 > 0)                     /* was a prev marker in old map */
      {
         fem_dist = -fem_dist;
         mal_dist = -mal_dist;
         for (i = Loc0; i < l0; ++i)
         {
            fem_dist += gendist (Marek[0][i]);
            mal_dist += gendist (Marek[1][i]);
         }
      }
      else
      {
         for (i = 1; i < l0; ++i)       /* old Loc0 =0 */
         {
            fem_dist += gendist (Marek[0][i]);
            mal_dist += gendist (Marek[1][i]);
         }
      }
      Rec0[0] = recomb (fem_dist);
      Rec0[1] = recomb (mal_dist);
      Loc0 = 0;
   }
   return;
}

void  set_markdat (int ***yy, int nl, int nc, int *locl2)
{
   int   i, j, c, cfound;
   int   old_marker, new_marker;
   int  *nx;

/* first determine marker columns amd reset trait indexes */

   old_marker = 0;
   new_marker = 0;
   nx = malloc ((nl + 1) * sizeof (int));

   while (new_marker < nl)
   {
      ++old_marker;
      if (locl2[old_marker] == (new_marker + 1))
      {
         ++new_marker;
         nx[new_marker] = old_marker;
      }
   }

/* Now copy marker data to correct column in y-matrix */

   for (i = 0; i < NObs; ++i)
   {
      if ((ObsIx[i] = get_posit (ObsNam[i])) == INVALID_INDEX)
         give_warn (ERA_mdp, ObsNam[i]);
      else
         for (j = 0; j < nl; ++j)
         {
            yy[0][j + 1][ObsIx[i]] = ObsMark[i][nx[j + 1] - 1][0];
            yy[1][j + 1][ObsIx[i]] = ObsMark[i][nx[j + 1] - 1][1];
         }
   }
   for (c = 0; c < nc; ++c)
   {
      cfound = 0;
      for (i = 0; i < NObs; ++i)
         if (ObsIx[i] != INVALID_INDEX
             && (nghd[ObsIx[i]].component) == (c + 1))
            cfound = 1;
      if (!cfound)
         give_warn (ERA_nmd, c + 1);
   }
   free (nx);
   return;
}
