
#include "traits.h"                     /* includes msample.h and others included therein */

/* see traits.h for specs and purpose of routines in this file.
   EAT March 2002                                                 */

int   set_eval_points (int nloc, int *loc_left, int *loc_rght,
                       double *rl0, double *rr0, double *rl1, double *rr1)
{
   int   ntp, k, ntf, sx;
   double *rl[2], *rr[2];

   printf ("\n Table of evaluation points ");
   fflush (stdout);
   printf ("\n  Pos  Intval   loc_left  male-rec fem-rec ");
   printf ("      loc_right  male-rec fem-rec ");

   rl[0] = rl0;
   rl[1] = rl1;
   rr[0] = rr0;
   rr[1] = rr1;

   ntp = 0;
   k = 1;
   ntf = 1;
   while (ntp < NTPos + nloc - 1)
   {
      ++ntp;
      if ((IntVal[k + 1] == IntVal[k]) || (ntf == 1))  /* next pos is in interva
                                                          l */
      {
         if (ntf != 1)
            ++k;                        /* last posn was not a marker */
         ntf = 0;
         loc_left[ntp] = IntVal[k];
         loc_rght[ntp] = IntVal[k] + 1;
         for (sx = FEM; sx <= MAL; ++sx)
         {
            if (loc_left[ntp] == 0)     /* there is no preceding marker      */
            {
               rl[sx][ntp] = 0.5;
               rr[sx][ntp] = RecVal[sx][k];
            }
            else if (loc_rght[ntp] <= nloc)
            {                           /*  we are in an interval  */
               rl[sx][ntp] = RecVal[sx][k];
               rr[sx][ntp] = (Marek[sx][IntVal[k]] - RecVal[sx][k])
                  / (1.0 - 2 * RecVal[sx][k]);
            }
            else if (loc_rght[ntp] == (nloc + 1))
            {                           /* after last marker */
               rl[sx][ntp] = RecVal[sx][k];
               rr[sx][ntp] = 0.5;
            }
         }
         if (loc_left[ntp] == 0)
            loc_left[ntp] = -1;
         if (loc_rght[ntp] == (nloc + 1))
            loc_rght[ntp] = -1;
      }
      else
         /* next position is a marker */
      {
         ntf = 1;
         ++k;
         loc_left[ntp] = IntVal[k];
         loc_rght[ntp] = IntVal[k];
         for (sx = FEM; sx <= MAL; ++sx)
         {
            rl[sx][ntp] = 0.0;
            rr[sx][ntp] = 0.0;
         }
      }
      printf ("\n  %d  %d  %d    %d  %f  %f    %d  %f  %f",
              ntp, k, ntf, loc_left[ntp], rl[1][ntp], rl[0][ntp],
              loc_rght[ntp], rr[1][ntp], rr[0][ntp]);
      fflush (stdout);
   }
   ++ntp;
   printf ("\n");

   return (ntp);
}

int   print_traits (int nt, int *loc_fl)
{
   int   i, kk, nm;
   nm = 0;
   printf (" Number of Traits (including marker traits) is %d\n", nt);
   printf (" trait index:  trait name:  trait type:   input index:  locus index \n");
   fflush (stdout);
   for (i = 0; i < nt; ++i)
   {
      printf (" %12d", i);
      printf ("  %12s ", traits[i].self);
      if (traits[i].trait_type == MARKER_GENO)
      {
         printf ("  geno-marker");
         ++nm;
      }
      else if (traits[i].trait_type == GENOTYPIC)
         printf ("  genotypic  ");
      else if (traits[i].trait_type == DISCRETE)
         printf ("  discrete   ");
      else if (traits[i].trait_type == ORDINAL)
         printf ("  ordinal    ");
      else if (traits[i].trait_type == CONTINUOUS)
         printf (" continuous  ");
      else
         printf (" ??????????  ");
      printf (" %12d ", traits[i].locus);
      kk = traits[i].locus;
      if (traits[i].trait_type == MARKER_GENO)
         traits[i].locus = loc_fl[kk];
      printf (" %12d \n", traits[i].locus);
   }
   printf (" Number of mapped genotypic markers specified is %d \n", nm);
   printf ("\n");
   fflush (stdout);
   return (nm);
}

void  recess2 (double *vv, const struct Trait *trait,
               const Ind * id, const Loclist * ll, double lam)
/* will become a penetrance routine for a discrete trait */
{
   return;
}

/* ****************** JUNK STUFF ABOUT LINKED LIST *******************
   Indlist *child_ptr;

   for (j = i + 1; j < limits[1] + 1 && parent[j][2] == parent[i][2]; ++j)
   {
   child_ptr->next = (Indlist *) malloc (sizeof (Indlist));
   alloc_chk (WHERE, child_ptr->next);
   child_ptr = child_ptr->next;
   child_ptr->current = &nghd[parent[j][0]];
   }
   }
   child_ptr->next = NULL;
   ***************************************************** */

void  default_traits (int nt)
/* set up default names and specs for traits */
{
   int   i;
   struct Trait *trait;
   char  ss[MAXNAMLEN];

   for (i = 0; i < nt; i++)
   {
      trait = traits + i;
      trait->index = i;
      trait->locus = i;
      if (i == 0)                       /* trait locus */
         if (TraitPheno)                /* phenotypic data */
         {
            trait->trait_type = DISCRETE;
            trait->penetrance = recess2;
            strncpy (trait->self, "trait-pheno", MAXNAMLEN);
            trait->self[MAXNAMLEN] = '\0';
         }
         else
         {
            trait->trait_type = GENOTYPIC;
            strncpy (trait->self, "trait-geno", MAXNAMLEN);
            trait->self[MAXNAMLEN] = '\0';
         }
      else
      {
         trait->trait_type = MARKER_GENO;
         sprintf (ss, "marker-%d", i);
         strncpy (trait->self, ss, MAXNAMLEN);
         trait->self[MAXNAMLEN] = '\0';
      }
   }
   return;
}

void  read_traits (int nt, FILE * pedF)
/* if ii=1, read trait names and loci etc.   */
{
   int   i, j, num;
   char  trec[MAX_RECSIZ + 1];
   struct Trait *trait;
   char  ss[MAXNAMLEN];
   int   trdat[4];

   i = 0;
   while (i < nt)
   {
      trait = traits + i;

      fscanf (pedF, "%d", trdat);
      trait->index = trdat[0];          /* index order is currently
                                           order in marker data file */
/* need three numbers -- index, locus, and data_index */

      read_name (pedF, ss);
      strncpy (trait->self, ss, MAXNAMLEN);

      for (j = 0; j < 4; ++j)           /* read other infor about trait i  */
         fscanf (pedF, "%d", trdat + j);

      trait->trait_type = trdat[0];
      trait->locus = trdat[2];          /* index in INPUT marker data, for now */
      /* will be reset later */
      ++i;
   }

   return;
}

void  flag_loci (int nt, int nloc, int *loc_fl)
/*  sets a trait flag for each marker locus, by input record */
/*  sets transform of loci for setting marker data           */
{
   int   i, j;

   for (i = 0; i <= nloc; ++i)
      loc_fl[i] = -1;

   for (i = 0; i < nt; ++i)
      if (traits[i].trait_type == MARKER_GENO)
         loc_fl[traits[i].locus] = i;

   j = 0;
   for (i = 0; i <= nloc; ++i)
      if (loc_fl[i] > -1)
      {
         ++j;
         loc_fl[i] = j;
      }

   return;
}

int   set_traits (int ii, int nl, int *loc_fl, FILE * pedF)
/* if ii=0, have no trait spec set -- default */
/* if ii=1, read trait names and specs        */
/* if ii=2 or more, then no 1-1 correspondence traits and loci */

{
   int   nt;                                /* number of traits, currently = number of loci */

   if (ii == 0)
      nt = nl + 1;
   else if (ii == 1)
      fscanf (pedF, "%d", &nt);

/* Allocate trait structures */

   traits = malloc (sizeof (struct Trait) * nt);
   alloc_chk (WHERE, traits);

/* set trait ; default case ii=0  */
   if (ii == 0)
      default_traits (nt);

/* ii=1; read traits names and specs  */
   else if (ii == 1)
      read_traits (nt, pedF);

/* flag marker loci that are defined by the traits  */
   flag_loci (nt, nl, loc_fl);

/* return the number of traits */
   return (nt);
}
void  free_traits (void)
{
   if (traits)
   {
      free (traits);
      traits = NULL;
   }
   return;
}
