#include "nghds.h"

/*****************************************************************
  Function to read pedigree data in from a file written in free format and
  to form a data matrix for the pedigree.
  This version uses the "genetic data columns" as fixed-effects indicators,
  and also reads in a quantitative data value "quant" -- becomes "x"
  The nghds() and position() routines are then called

  Various checks on pedigree size etc are carried out. The global variables
  "pedsize" and "n_compnts" are also evaluated here.

  Modified out of input_data() in Quant library;  6/8/93. (EAT).

*****************************************************************/

void  input_ped (FILE * fp, int nn, int **dodo, double **quant)
{
   int   i, j, k, n;
   KEYS_PP ppK;

   if (NReals && !quant)
      give_up (ERR_qnt, "input_ped");

   for (i = 0; i < PedSize; ++i)
   {
      /* Skip the first three columns (the names are already in ppKeyTab) */
      for (j = 0; j < NNams; ++j)
         fscanf (fp, "%*s");

      data[i][0] = i;
      /* the ix in the keytable starts from 1 */
      data[i][1] = i + 1;               /* index of i + 1 */
      ppK = ppKeyTab + i;
      data[i][2] = (*ppK)->ixPa;        /* index of father */
      data[i][3] = (*ppK)->ixMa;        /* index of mother */

      /* read in the rest of the data */
      for (j = 0; j < NInts; ++j)
      {
         k = SelInt[j + NNams];
         if (k)
            n = fscanf (fp, "%d", &data[i][k]);
         else
            n = fscanf (fp, "%*d");
      }
      for (j = 0, k = 0; j < NReals; ++j)
         if (SelTrait[j] == TRUE)
         {
            ++k;
            n = fscanf (fp, "%lf", &quant[k - 1][i]);
         }
         else
            n = fscanf (fp, "%*f");

      for (j = 0; j < nn; ++j)
         n = fscanf (fp, "%d", &dodo[j][i]);
      if (n == EOF)
         break;
   }

   fprintf (stdout, " Pedigree data file records:  names");
   if (NInts > 0)
      fprintf (stdout, ", %d integer%s", NInts, NInts > 1 ? "s" : "");
   if (NReals > 0)
      fprintf (stdout, ", %d real number%s", NReals, NReals > 1 ? "s" : "");
   fprintf (stdout, "\n\n");

   fprintf (stdout, "% d individuals read\n\n", i);
   fflush (stdout);

   if (i < PedSize)
   {
      give_warn (ERR_nex, PedSize);
      if (i < 3)
         give_up (ERR_siz, "input_ped");
   }
   else
   {
      n = fscanf (fp, "%*s");
      if (n != EOF)
         give_warn (ERR_urd, "input_ped");
   }
   pedsize = i;

   /* Now set neighbourhoods */

   nghds (data);
   cmplist = (Component *) malloc (MaxNCmps * sizeof (Component));
   alloc_chk (WHERE, cmplist);

   n_compnts = components ();

   if (n_compnts > 1)
      fprintf (stdout, " %d components found\n\n"
               " %d members are in the largest component\n\n",
               n_compnts, max_cpsize);
   return;
}
