/* vim: sw=3
 * $Id: proc_shnl.c,v 1.6 2002/04/26 23:51:45 nali Exp $ */

#include "parseauto.h"

int
      proc_shnl (void)
{
   int   i;
   int   lnkd, m_no, entry, sex;
   int   sxsp_map = FALSE;

   SIM_PTR p_s = 0;
   DIST_PTR p_d = 0;
   UNLNKD_PTR p_u = 0;
   TLIST_PTR p_t = 0;
   VMAP_PTR p_v = 0;
   PRNTMAP_PTR p_m = 0;
   PRNTLOC_PTR p_l = 0;

   /*                                    * checks for select statement       */

   if (!test_atz_sel ())
      if (proc_err ())
         return FALSE;
   if (n_err)
      return proc_end ();

   entry = FALSE;
   p_s = nxt_sim_chrom (&entry);        /* selects one chromosome only       *

                                         * checks marker map & trait map     *
                                         * (sex-averaged & sex-specific)     */
   if (!test_mark_map (p_s, &sxsp_map))
      if (proc_err ())
         return FALSE;
   if (!test_trait_vmap (p_s, &sxsp_map))
      if (proc_err ())
         return FALSE;
   if (n_err)                           /* if errors, quit                   */
      return proc_end ();

   /*                                    * adds unlinked trait to trait list */
   lnkd = FALSE;                        /* (lm_auto)                         */
   entry = FALSE;
   if (p_u = nxt_unlnkd_trait (&entry))
      if (!test_trait_list (lnkd, p_u->id))
         if (proc_err ())
            return FALSE;

   /*                                    * adds trait with varying positions */
   lnkd = FALSE;                        /* to trait list (lm_lods, schnell)  */
   if (p_v = nxt_trait_vmap (FEM, p_s))
      if (!test_trait_list (lnkd, p_v->id))
         if (proc_err ())
            return FALSE;
   /*                                    * for each marker,                  */
   entry = FALSE;                       /* check allele frequencies          */
   while (m_no = nxt_mark (&entry, p_s))
      if (!test_mark_loci (p_s, m_no))
         if (proc_err ())
            return FALSE;
   /*                                    * for each trait,                   */
   /*                                    * checks variances, genotype        */
   entry = FALSE;                       /* means and allele frequencies      */
   while (p_t = nxt_trait (&entry))
   {
      if (!test_trait_vars (p_t->id))
         if (proc_err ())
            return FALSE;
      if (!test_trait_genotyp (p_t->id))
         if (proc_err ())
            return FALSE;
      if (!test_trait_loci (p_t->id))
         if (proc_err ())
            return FALSE;
   }
   if (n_err)
      return proc_end ();
   /*                                    * if no errors, prints out params   */
   for (sex = FEM; sex <= AVG; ++sex)
   {                                    /* first assigns loci to sex-specific
                                         * print maps
                                         * defaults to sex-averaged,  then
                                         * to F maps, for maps not present   */

      /*                                 * puts markers into map             */
      p_m = add_prntmap (p_s, sex);

      if (p_s->no_mark == 1)
         add_only_mark_locus (p_m);
      else
      {
         entry = FALSE;
         while (p_d = nxt_mapd_mark (&entry, sex, p_s))
            add_mark_locus (p_m, p_d);
      }

      /* find distance between adjacent loci */
      entry = FALSE;
      while ((p_l = nxt_prntmap_locus (&entry, p_m)))
         fix_locus_dist (p_m, p_l);

      if ((sxsp_map && (sex == FEM || sex == MAL))
          || (!sxsp_map && sex == AVG))
      {
         prnt_map (p_m, sex);           /* prints the map                    */
         /*                              * prints the trait intervals        */
         if (p_v = nxt_trait_vmap (sex, p_s))  /* (lm_lods)                  */
            prnt_vmap (p_v, sex);
      }
   }

   prnt_mark_freqs (p_s);               /* prints marker allele freqs        */

   prnt_trait_freqs (p_m);              /* prints trait allele freqs         */

   prnt_unlnkd_freqs ();

   prnt_vtrait_freqs (p_s);

   fprintf (stdout, "\n");

   if (n_err)
      return proc_end ();

   NMarks = p_m->no_mark;               /* counts loci                       */

   if (p_v)
      ++NVary;

   NTraits = NLnkd + NUnlnkd + NVary;

   NLoci = NMarks + NTraits;

   alloc_marek ();                      /* stores marker recomb fracts in
                                         * marek[][]                         */
   mak_marek ();

   if (NVary)                           /* sets NTPos, the no. of trait pos  */
   {                                    /* (arrays arrays are 1 greater)     */
      entry = FALSE;                    /* and recomb fracts in RecVal[][]
                                         * and positions in IntVal[]         */

      NTExtPos = p_v->no_xtrn;          /* ADDED BY AWG for lm_bayes         */
      NTIntPos = p_v->no_prop;          /* END OF ADDED SECTION              */

      NTPos = 2 * p_v->no_xtrn + (NMarks - 1) * p_v->no_prop + 1;

      alloc_intval ();

      mak_intval (p_s);

      mak_recval (p_s);
   }

   for (i = 0; i < NInts; ++i)
      SelInt[i] = i + 1;
   NTraits = NReals;
   SelTrait[0] = 1;

   /*                                    * for each locus, stores            */
   entry = FALSE;                       /* allele frequencies in Qq[][]      */
   p_m = nxt_prntmap (&entry, FEM);
   entry = FALSE;
   while (p_l = nxt_prntmap_locus (&entry, p_m))
      mak_allefrq (p_m, p_l, p_l->id);
   if (p_u)
      mak_allefrq (NULL, NULL, p_u->id);
   mak_allefrq (NULL, NULL, p_v->id);

   if (!test_markdat (p_s))             /* checks marker data                */
      if (proc_err ())
         return FALSE;

   if (!n_err)
   {
      mak_markdat (p_s);                /* stores marker data                */
      prnt_markdat (p_s);
   }

   /* save trait parameters (means, variances) */
   if (NTraits > 0)
   {
      alloc_trait ();
      mak_trait (1, 0, 0);
   }

   if (!test_atz_pars ())               /* checks computational parms        */
      if (proc_err ())
         return FALSE;

   /* FIXME */
   /* checks input parmeters            */
/*    if (!test_atz_inpt ())         */
/*       if (proc_err ())  */
/*          return FALSE;  */

   if (n_err)
      return proc_end ();

   TraitPheno = T;

   prnt_atz_pars ();                    /* prints program parameters         */

   mak_atz_inpt ();                     /* set up stuff for reading          */

   return TRUE;
}
