/*************************************************************************
  
  program name: quarz.c
  
  function to compute quadratic z' * G_inv * z , using a matrix stored as
      its rows/columns/values of non-zero triangular terms (usually lower
      triangular terms)
  
  the routine replaces the routine "bigprod" in bivar.c
  
  Working and installed in Quant; 8/24/91
**************************************************************************
  
  routine quizza.c 
	does the same computation, for non-inbred pedigree, using
		pedigree structure instead of matrix
	implemented 2/94  (EAT).
  
**************************************************************************/

#include "quant.h"

double quarz (double *xx1, double *xx2, Term * pp, int n)
/*
 * Computes x1'pp x2 where pp is a symmetric matrix, whose 
 * non-zero triangular terms are specified. 
 */
{
   int   i;
   double cc = 0.0;
   for (i = 1; i <= n; ++i)
   {
      cc += xx1[pp[i].r1] * xx2[pp[i].r2] * pp[i].value;
      if (pp[i].r1 != pp[i].r2)
         cc += xx1[pp[i].r2] * xx2[pp[i].r1] * pp[i].value;
   }
   return (cc);
}

double quizza (double *zz)
{
   int   i;
   Ind  *dad, *mum;
   double lnc, zm, zf, zd;

   lnc = 0.0;

   for (i = 0; i < pedsize; ++i)
   {
      dad = nghd[i].pa;
      if (isNullNode (dad))
         lnc += zz[i] * zz[i];
      else
      {
         mum = nghd[i].ma;
         zm = zz[dad->index];
         zf = zz[mum->index];
         zd = (zz[i] - (zm + zf) * 0.5);
         lnc += 2.0 * zd * zd;
      }
   }
   return (lnc);
}
